@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2006-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -30,9 +30,10 @@ AnimationParticle::AnimationParticle(Map *map, Animation *animation):
 {
 }
 
-AnimationParticle::AnimationParticle(Map *map, xmlNodePtr animationNode):
+AnimationParticle::AnimationParticle(Map *map, xmlNodePtr animationNode,
+                                     const std::string& dyePalettes):
     ImageParticle(map, 0),
-    mAnimation(new SimpleAnimation(animationNode))
+    mAnimation(new SimpleAnimation(animationNode, dyePalettes))
 {
 }
 
