@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2006-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -35,7 +35,8 @@ class AnimationParticle : public ImageParticle
     public:
         AnimationParticle(Map *map, Animation *animation);
 
-        AnimationParticle(Map *map, xmlNodePtr animationNode);
+        AnimationParticle(Map *map, xmlNodePtr animationNode,
+                          const std::string& dyePalettes = std::string());
 
         ~AnimationParticle();
 
