@@ -1,6 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2008  The Mana World Development Team
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -20,10 +21,6 @@
 
 #include "avatar.h"
 
-#include "localplayer.h"
-
-#include <sstream>
-
 Avatar::Avatar(const std::string &name):
         mName(name),
         mHp(0), mMaxHp(0),
