@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -26,6 +26,7 @@
 #include <string>
 
 class Channel;
+class ChatWindow;
 
 class ChannelManager
 {
@@ -40,6 +41,7 @@ public:
     void removeChannel(Channel *channel);
 
 private:
+    friend class ChatWindow;
     std::list<Channel*> mChannels;
 };
 
