@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -22,8 +22,10 @@
 #include "client.h"
 #include "main.h"
 
+#include "chatlogger.h"
 #include "configuration.h"
 #include "emoteshortcut.h"
+#include "event.h"
 #include "game.h"
 #include "itemshortcut.h"
 #include "keyboardconfig.h"
@@ -40,17 +42,15 @@
 #include "gui/charselectdialog.h"
 #include "gui/connectiondialog.h"
 #include "gui/gui.h"
-#include "gui/login.h"
+#include "gui/logindialog.h"
 #include "gui/okdialog.h"
 #include "gui/quitdialog.h"
 #include "gui/register.h"
 #include "gui/sdlinput.h"
 #include "gui/serverdialog.h"
 #include "gui/setup.h"
-#include "gui/theme.h"
 #include "gui/unregisterdialog.h"
-#include "gui/updatewindow.h"
-#include "gui/userpalette.h"
+#include "gui/updaterwindow.h"
 #include "gui/worldselectdialog.h"
 
 #include "gui/widgets/button.h"
@@ -64,13 +64,16 @@
 #include "net/net.h"
 #include "net/worldinfo.h"
 
-#include "resources/colordb.h"
+#include "resources/hairdb.h"
 #include "resources/emotedb.h"
 #include "resources/image.h"
 #include "resources/itemdb.h"
 #include "resources/monsterdb.h"
+#include "resources/specialdb.h"
 #include "resources/npcdb.h"
 #include "resources/resourcemanager.h"
+#include "resources/theme.h"
+#include "resources/userpalette.h"
 
 #include "utils/gettext.h"
 #include "utils/mkdir.h"
@@ -83,7 +86,7 @@
 #include <physfs.h>
 #include <SDL_image.h>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <SDL_syswm.h>
 #include "utils/specialfolder.h"
 #else
@@ -104,24 +107,22 @@ static const int defaultMusicVolume = 60;
 
 // TODO: Get rid fo these globals
 std::string errorMessage;
-ErrorListener errorListener;
 LoginData loginData;
 
 Configuration config;         /**< XML file configuration reader */
 Configuration branding;       /**< XML branding information reader */
 Configuration paths;          /**< XML default paths information reader */
 Logger *logger;               /**< Log object */
+ChatLogger *chatLogger;       /**< Chat log object */
 KeyboardConfig keyboard;
 
 UserPalette *userPalette;
 Graphics *graphics;
 
-Sound sound;
+ItemDB *itemDb;               /**< Items info database */
+HairDB hairDB;                /**< Hair styles and colors info database */
 
-void ErrorListener::action(const gcn::ActionEvent &)
-{
-    Client::setState(STATE_CHOOSE_SERVER);
-}
+Sound sound;
 
 volatile int tick_time;       /**< Tick counter */
 volatile int fps = 0;         /**< Frames counted in the last second */
@@ -153,55 +154,45 @@ Uint32 nextSecond(Uint32 interval, void *param)
     return interval;
 }
 
-/**
- * @return the elapsed time in milliseconds
- * between two tick values.
- */
-int get_elapsed_time(int start_time)
+int get_elapsed_time(int startTime)
 {
-    if (start_time <= tick_time)
-        return (tick_time - start_time) * MILLISECONDS_IN_A_TICK;
+    if (startTime <= tick_time)
+        return (tick_time - startTime) * MILLISECONDS_IN_A_TICK;
     else
-        return (tick_time + (MAX_TICK_VALUE - start_time))
+        return (tick_time + (MAX_TICK_VALUE - startTime))
                 * MILLISECONDS_IN_A_TICK;
 }
 
-
-// This anonymous namespace hides whatever is inside from other modules.
-namespace {
-
-class AccountListener : public gcn::ActionListener
+bool isDoubleClick(int selected)
 {
-public:
-    void action(const gcn::ActionEvent &)
-    {
-        Client::setState(STATE_CHAR_SELECT);
-    }
-} accountListener;
+    const Uint32 maximumDelay = 500;
+    static Uint32 lastTime = 0;
+    static int lastSelected = -1;
 
-class LoginListener : public gcn::ActionListener
-{
-public:
-    void action(const gcn::ActionEvent &)
+    if (selected == lastSelected && lastTime + maximumDelay >= SDL_GetTicks())
     {
-        Client::setState(STATE_LOGIN);
+        lastTime = 0;
+        return true;
     }
-} loginListener;
 
-} // anonymous namespace
+    lastTime = SDL_GetTicks();
+    lastSelected = selected;
+    return false;
+}
 
 
 Client *Client::mInstance = 0;
 
 Client::Client(const Options &options):
     mOptions(options),
-    mRootDir(""),
+    mGame(0),
     mCurrentDialog(0),
     mQuitDialog(0),
     mDesktop(0),
     mSetupButton(0),
     mState(STATE_CHOOSE_SERVER),
     mOldState(STATE_START),
+    mStateAfterOkDialog(mState),
     mIcon(0),
     mLogicCounterId(0),
     mSecondsCounterId(0),
@@ -212,6 +203,11 @@ Client::Client(const Options &options):
 
     logger = new Logger;
 
+    // Set default values for configuration files
+    branding.setDefaultValues(getBrandingDefaults());
+    paths.setDefaultValues(getPathsDefaults());
+    config.setDefaultValues(getConfigDefaults());
+
     // Load branding information
     if (!options.brandingPath.empty())
     {
@@ -222,9 +218,15 @@ Client::Client(const Options &options):
     initHomeDir();
     initConfiguration();
 
+    chatLogger = new ChatLogger;
+    if (options.chatLogDir.empty())
+        chatLogger->setLogDir(mLocalDataDir + std::string("/logs/"));
+    else
+        chatLogger->setLogDir(options.chatLogDir);
+
     // Configure logger
     logger->setLogFile(mLocalDataDir + std::string("/mana.log"));
-    logger->setLogToStandardOut(config.getValue("logToStandardOut", 0));
+    logger->setLogToStandardOut(config.getBoolValue("logToStandardOut"));
 
     // Log the mana version
     logger->log("Mana %s", FULL_VERSION);
@@ -281,7 +283,7 @@ Client::Client(const Options &options):
         std::string path = options.brandingPath;
 
         // Strip blah.mana from the path
-#ifdef WIN32
+#ifdef _WIN32
         int loc1 = path.find_last_of('/');
         int loc2 = path.find_last_of('\\');
         int loc = std::max(loc1, loc2);
@@ -300,14 +302,14 @@ Client::Client(const Options &options):
     resman->addToSearchPath(mLocalDataDir, false);
 
     std::string iconFile = branding.getValue("appIcon", "icons/mana");
-#ifdef WIN32
+#ifdef _WIN32
     iconFile += ".ico";
 #else
     iconFile += ".png";
 #endif
     iconFile = resman->getPath(iconFile);
     logger->log("Loading icon from file: %s", iconFile.c_str());
-#ifdef WIN32
+#ifdef _WIN32
     static SDL_SysWMinfo pInfo;
     SDL_GetWMInfo(&pInfo);
     // Attempt to load icon from .ico file
@@ -347,11 +349,11 @@ Client::Client(const Options &options):
     graphics = new Graphics;
 #endif
 
-    const int width = (int) config.getValue("screenwidth", defaultScreenWidth);
-    const int height = (int) config.getValue("screenheight", defaultScreenHeight);
+    const int width = config.getIntValue("screenwidth");
+    const int height = config.getIntValue("screenheight");
     const int bpp = 0;
-    const bool fullscreen = ((int) config.getValue("screen", 0) == 1);
-    const bool hwaccel = ((int) config.getValue("hwaccel", 0) == 1);
+    const bool fullscreen = config.getBoolValue("screen");
+    const bool hwaccel = config.getBoolValue("hwaccel");
 
     // Try to set the desired video mode
     if (!graphics->setVideoMode(width, height, bpp, fullscreen, hwaccel))
@@ -374,13 +376,11 @@ Client::Client(const Options &options):
     // Initialize sound engine
     try
     {
-        if (config.getValue("sound", 0) == 1)
+        if (config.getBoolValue("sound"))
             sound.init();
 
-        sound.setSfxVolume((int) config.getValue("sfxVolume",
-                    defaultSfxVolume));
-        sound.setMusicVolume((int) config.getValue("musicVolume",
-                    defaultMusicVolume));
+        sound.setSfxVolume(config.getIntValue("sfxVolume"));
+        sound.setMusicVolume(config.getIntValue("musicVolume"));
     }
     catch (const char *err)
     {
@@ -398,32 +398,32 @@ Client::Client(const Options &options):
     userPalette = new UserPalette;
     setupWindow = new Setup;
 
-    sound.playMusic(branding.getValue("loginMusic", "Magick - Real.ogg"));
+    sound.playMusic(branding.getStringValue("loginMusic"));
 
     // Initialize default server
     mCurrentServer.hostname = options.serverName;
     mCurrentServer.port = options.serverPort;
     loginData.username = options.username;
     loginData.password = options.password;
-    loginData.remember = config.getValue("remember", 0);
+    loginData.remember = config.getBoolValue("remember");
     loginData.registerLogin = false;
 
     if (mCurrentServer.hostname.empty())
-    {
-        mCurrentServer.hostname = branding.getValue("defaultServer",
-                                                    "").c_str();
-    }
+        mCurrentServer.hostname = branding.getValue("defaultServer","").c_str();
 
     if (mCurrentServer.port == 0)
     {
         mCurrentServer.port = (short) branding.getValue("defaultPort",
-                                                       DEFAULT_PORT);
+                                                                  DEFAULT_PORT);
         mCurrentServer.type = ServerInfo::parseType(
-                branding.getValue("defaultServerType", "tmwathena"));
+                           branding.getValue("defaultServerType", "tmwathena"));
     }
 
+    if (chatLogger)
+        chatLogger->setServerName(mCurrentServer.hostname);
+
     if (loginData.username.empty() && loginData.remember)
-        loginData.username = config.getValue("username", "");
+        loginData.username = config.getStringValue("username");
 
     if (mState != STATE_ERROR)
         mState = STATE_CHOOSE_SERVER;
@@ -435,8 +435,16 @@ Client::Client(const Options &options):
 
     // Initialize frame limiting
     SDL_initFramerate(&mFpsManager);
-    config.addListener("fpslimit", this);
-    optionChanged("fpslimit");
+
+    listen(Event::ConfigChannel);
+
+    //TODO: fix having to fake a option changed event
+    Event fakeevent(Event::ConfigOptionChanged);
+    fakeevent.setString("option", "fpslimit");
+    event(Event::ConfigChannel, fakeevent);
+
+    // Initialize PlayerInfo
+    PlayerInfo::init();
 }
 
 Client::~Client()
@@ -445,9 +453,9 @@ Client::~Client()
     SDL_RemoveTimer(mSecondsCounterId);
 
     // Unload XML databases
-    ColorDB::unload();
+    hairDB.unload();
     EmoteDB::unload();
-    ItemDB::unload();
+    delete itemDb;
     MonsterDB::unload();
     NPCDB::unload();
     StatusEffect::unload();
@@ -483,15 +491,14 @@ int Client::exec()
 {
     int lastTickTime = tick_time;
 
-    Game *game = 0;
     SDL_Event event;
 
     while (mState != STATE_EXIT)
     {
-        if (game)
+        if (mGame)
         {
             // Let the game handle the events while it is active
-            game->handleInput();
+            mGame->handleInput();
         }
         else
         {
@@ -506,6 +513,10 @@ int Client::exec()
 
                     case SDL_KEYDOWN:
                         break;
+
+                    case SDL_VIDEORESIZE:
+                        handleVideoResize(event.resize.w, event.resize.h);
+                        break;
                 }
 
                 guiInput->pushInput(event);
@@ -518,8 +529,10 @@ int Client::exec()
         while (get_elapsed_time(lastTickTime) > 0)
         {
             gui->logic();
-            if (game)
-                game->logic();
+            if (mGame)
+                mGame->logic();
+
+            sound.logic();
 
             ++lastTickTime;
         }
@@ -568,17 +581,14 @@ int Client::exec()
 
             mDesktop = new Desktop;
             top->add(mDesktop);
-            mSetupButton = new Button(_("Setup"), "Setup", this);
+            mSetupButton = new Button("", "Setup", this);
+            mSetupButton->setButtonPopupText(_("Setup"));
+            mSetupButton->setButtonIcon("button-icon-setup.png");
             mSetupButton->setPosition(top->getWidth() - mSetupButton->getWidth()
                                      - 3, 3);
             top->add(mSetupButton);
 
-            int screenWidth = (int) config.getValue("screenwidth",
-                                                    defaultScreenWidth);
-            int screenHeight = (int) config.getValue("screenheight",
-                                                     defaultScreenHeight);
-
-            mDesktop->setSize(screenWidth, screenHeight);
+            mDesktop->setSize(graphics->getWidth(), graphics->getHeight());
         }
 
         if (mState == STATE_SWITCH_LOGIN && mOldState == STATE_GAME)
@@ -588,14 +598,17 @@ int Client::exec()
 
         if (mState != mOldState)
         {
-            Net::GeneralHandler *generalHandler = Net::getGeneralHandler();
-            if (generalHandler)
-                generalHandler->stateChanged(mOldState, mState);
+            {
+                Event event(Event::StateChange);
+                event.setInt("oldState", mOldState);
+                event.setInt("newState", mState);
+                event.trigger(Event::ClientChannel);
+            }
 
             if (mOldState == STATE_GAME)
             {
-                delete game;
-                game = 0;
+                delete mGame;
+                mGame = 0;
             }
 
             mOldState = mState;
@@ -748,16 +761,43 @@ int Client::exec()
                     // Read default paths file 'data/paths.xml'
                     paths.init("paths.xml", true);
 
+                    Event::trigger(Event::ClientChannel, Event::LoadingDatabases);
+
                     // Load XML databases
-                    ColorDB::load();
-                    ItemDB::load();
-                    Being::load(); // Hairstyles
+                    hairDB.load();
+                    switch (Net::getNetworkType())
+                    {
+                      case ServerInfo::TMWATHENA:
+                        itemDb = new TmwAthena::TaItemDB;
+                      break;
+                      case ServerInfo::MANASERV:
+                        itemDb = new ManaServ::ManaServItemDB;
+                      break;
+                      default:
+                        // Nothing
+                        itemDb = 0;
+                      break;
+                    }
+                    if (!itemDb || !itemDb->isLoaded())
+                    {
+                        // Warn and return to login screen
+                        errorMessage =
+                            _("This server is missing needed world data. "
+                              "Please contact the administrator(s).");
+                        showOkDialog(_("ItemDB: Error while loading "
+                                       ITEMS_DB_FILE "!"), errorMessage,
+                                     STATE_CHOOSE_SERVER);
+                        break;
+                    }
                     MonsterDB::load();
+                    SpecialDB::load();
                     NPCDB::load();
                     EmoteDB::load();
                     StatusEffect::load();
                     Units::loadUnits();
 
+                    ActorSprite::load();
+
                     mDesktop->reloadWallpaper();
 
                     mState = STATE_GET_CHARACTERS;
@@ -783,12 +823,16 @@ int Client::exec()
                             mOptions.character, CharSelectDialog::Choose))
                     {
                         ((CharSelectDialog*) mCurrentDialog)->selectByName(
-                                config.getValue("lastCharacter", ""),
+                                config.getStringValue("lastCharacter"),
                                 mOptions.chooseDefault ?
                                     CharSelectDialog::Choose :
                                     CharSelectDialog::Focus);
                     }
 
+                    // Choosing character on the command line should work only
+                    // once, clear it so that 'switch character' works.
+                    mOptions.character.clear();
+
                     break;
 
                 case STATE_CONNECT_GAME:
@@ -812,8 +856,8 @@ int Client::exec()
 
                 case STATE_GAME:
                     logger->log("Memorizing selected character %s",
-                            player_node->getName().c_str());
-                    config.setValue("lastCharacter", player_node->getName());
+                            local_player->getName().c_str());
+                    config.setValue("lastCharacter", local_player->getName());
 
                     // Fade out logon-music here too to give the desired effect
                     // of "flowing" into the game.
@@ -830,21 +874,17 @@ int Client::exec()
                     mCurrentDialog = NULL;
 
                     logger->log("State: GAME");
-                    game = new Game;
+                    mGame = new Game;
                     break;
 
                 case STATE_LOGIN_ERROR:
                     logger->log("State: LOGIN ERROR");
-                    mCurrentDialog = new OkDialog(_("Error"), errorMessage);
-                    mCurrentDialog->addActionListener(&loginListener);
-                    mCurrentDialog = NULL; // OkDialog deletes itself
+                    showErrorDialog(errorMessage, STATE_LOGIN);
                     break;
 
                 case STATE_ACCOUNTCHANGE_ERROR:
                     logger->log("State: ACCOUNT CHANGE ERROR");
-                    mCurrentDialog = new OkDialog(_("Error"), errorMessage);
-                    mCurrentDialog->addActionListener(&accountListener);
-                    mCurrentDialog = NULL; // OkDialog deletes itself
+                    showErrorDialog(errorMessage, STATE_CHAR_SELECT);
                     break;
 
                 case STATE_REGISTER_PREP:
@@ -862,6 +902,7 @@ int Client::exec()
                 case STATE_REGISTER_ATTEMPT:
                     logger->log("Username is %s", loginData.username.c_str());
                     Net::getLoginHandler()->registerAccount(&loginData);
+                    loginData.password = "";
                     break;
 
                 case STATE_CHANGEPASSWORD:
@@ -878,11 +919,10 @@ int Client::exec()
 
                 case STATE_CHANGEPASSWORD_SUCCESS:
                     logger->log("State: CHANGE PASSWORD SUCCESS");
-                    mCurrentDialog = new OkDialog(_("Password Change"),
-                            _("Password changed successfully!"));
-                    mCurrentDialog->addActionListener(&accountListener);
-                    mCurrentDialog = NULL; // OkDialog deletes itself
-                    loginData.password = loginData.newPassword;
+                    showOkDialog(_("Password Change"),
+                                 _("Password changed successfully!"),
+                                 STATE_CHAR_SELECT);
+                    loginData.password = "";
                     loginData.newPassword = "";
                     break;
 
@@ -898,10 +938,9 @@ int Client::exec()
 
                 case STATE_CHANGEEMAIL_SUCCESS:
                     logger->log("State: CHANGE EMAIL SUCCESS");
-                    mCurrentDialog = new OkDialog(_("Email Change"),
-                            _("Email changed successfully!"));
-                    mCurrentDialog->addActionListener(&accountListener);
-                    mCurrentDialog = NULL; // OkDialog deletes itself
+                    showOkDialog(_("Email Change"),
+                                 _("Email changed successfully!"),
+                                 STATE_CHAR_SELECT);
                     break;
 
                 case STATE_UNREGISTER:
@@ -919,12 +958,10 @@ int Client::exec()
                     logger->log("State: UNREGISTER SUCCESS");
                     Net::getLoginHandler()->disconnect();
 
-                    mCurrentDialog = new OkDialog(_("Unregister Successful"),
-                            _("Farewell, come back any time..."));
+                    showOkDialog(_("Unregister Successful"),
+                                 _("Farewell, come back any time..."),
+                                 STATE_CHOOSE_SERVER);
                     loginData.clear();
-                    //The errorlistener sets the state to STATE_CHOOSE_SERVER
-                    mCurrentDialog->addActionListener(&errorListener);
-                    mCurrentDialog = NULL; // OkDialog deletes itself
                     break;
 
                 case STATE_SWITCH_SERVER:
@@ -977,9 +1014,7 @@ int Client::exec()
                 case STATE_ERROR:
                     logger->log("State: ERROR");
                     logger->log("Error: %s", errorMessage.c_str());
-                    mCurrentDialog = new OkDialog(_("Error"), errorMessage);
-                    mCurrentDialog->addActionListener(&errorListener);
-                    mCurrentDialog = NULL; // OkDialog deletes itself
+                    showErrorDialog(errorMessage, STATE_CHOOSE_SERVER);
                     Net::getGameHandler()->disconnect();
                     break;
 
@@ -993,12 +1028,32 @@ int Client::exec()
     return 0;
 }
 
-void Client::optionChanged(const std::string &name)
+void Client::showOkDialog(const std::string &title,
+                          const std::string &message,
+                          State state)
 {
-    const int fpsLimit = (int) config.getValue("fpslimit", 60);
-    mLimitFps = fpsLimit > 0;
-    if (mLimitFps)
-        SDL_setFramerate(&mFpsManager, fpsLimit);
+    OkDialog *okDialog = new OkDialog(title, message);
+    okDialog->addActionListener(this);
+    mStateAfterOkDialog = state;
+}
+
+void Client::showErrorDialog(const std::string &message, State state)
+{
+    showOkDialog(_("Error"), message, state);
+}
+
+void Client::event(Event::Channel channel, const Event &event)
+{
+    if (channel == Event::ConfigChannel &&
+        event.getType() == Event::ConfigOptionChanged &&
+        event.getString("option") == "fpslimit")
+    {
+        const int fpsLimit = config.getIntValue("fpslimit");
+        mLimitFps = fpsLimit > 0;
+        if (mLimitFps)
+            SDL_setFramerate(&mFpsManager, fpsLimit);
+    }
+
 }
 
 void Client::action(const gcn::ActionEvent &event)
@@ -1014,12 +1069,16 @@ void Client::action(const gcn::ActionEvent &event)
         if (window->isVisible())
             window->requestMoveToTop();
     }
+
+    // If this came from the OkDialog used by showOkDialog
+    if (event.getId() == "ok")
+        mState = mStateAfterOkDialog;
 }
 
 void Client::initRootDir()
 {
     mRootDir = PHYSFS_getBaseDir();
-#ifdef WIN32
+#ifdef _WIN32
     std::string portableName = mRootDir + "portable.xml";
     struct stat statbuf;
 
@@ -1082,7 +1141,10 @@ void Client::initHomeDir()
         mLocalDataDir = std::string(PHYSFS_getUserDir()) +
             "/Library/Application Support/" +
             branding.getValue("appName", "Mana");
-#elif defined WIN32
+#elif defined __HAIKU__
+        mLocalDataDir = std::string(PHYSFS_getUserDir()) +
+           "/config/data/Mana";
+#elif defined _WIN32
         mLocalDataDir = getSpecialFolderLocation(CSIDL_LOCAL_APPDATA);
         if (mLocalDataDir.empty())
             mLocalDataDir = std::string(PHYSFS_getUserDir());
@@ -1106,7 +1168,11 @@ void Client::initHomeDir()
         const std::string app = branding.getValue("appShort", "manasource");
 #ifdef __APPLE__
         mConfigDir = mLocalDataDir + "/" + app;
-#elif defined WIN32
+#elif defined __HAIKU__
+        mConfigDir = std::string(PHYSFS_getUserDir()) +
+           "/config/settings/Mana" +
+           branding.getValue("appName", "manasource");
+#elif defined _WIN32
         mConfigDir = getSpecialFolderLocation(CSIDL_APPDATA);
         if (mConfigDir.empty())
             mConfigDir = mLocalDataDir;
@@ -1209,47 +1275,31 @@ void Client::initConfiguration()
  */
 void Client::initUpdatesDir()
 {
-    std::stringstream updates;
-
     // If updatesHost is currently empty, fill it from config file
     if (mUpdateHost.empty())
-    {
-        mUpdateHost = config.getValue("updatehost", "");
-    }
+        mUpdateHost = config.getStringValue("updatehost");
 
-    // Don't go out of range int he next check
-    if (mUpdateHost.length() < 2)
+    // Exit on empty update host.
+    if (mUpdateHost.empty())
         return;
 
-    // Remove any trailing slash at the end of the update host
-    if (mUpdateHost.at(mUpdateHost.size() - 1) == '/')
-        mUpdateHost.resize(mUpdateHost.size() - 1);
+    logger->log("Setting update host: %s", mUpdateHost.c_str());
 
-    // Parse out any "http://" or "ftp://", and set the updates directory
-    size_t pos;
-    pos = mUpdateHost.find("://");
-    if (pos != mUpdateHost.npos)
+    std::string updateHost = getHostNameFromURL(mUpdateHost);
+
+    // Exit on a wrong update host.
+    if (updateHost.length() < 2)
     {
-        if (pos + 3 < mUpdateHost.length())
-        {
-            updates << "updates/" << mUpdateHost.substr(pos + 3);
-            mUpdatesDir = updates.str();
-        }
-        else
-        {
-            logger->log("Error: Invalid update host: %s", mUpdateHost.c_str());
-            errorMessage = strprintf(_("Invalid update host: %s"),
-                                     mUpdateHost.c_str());
-            mState = STATE_ERROR;
-        }
-    }
-    else
-    {
-        logger->log("Warning: no protocol was specified for the update host");
-        updates << "updates/" << mUpdateHost;
-        mUpdatesDir = updates.str();
+        // Show the original updateHostname in the error message.
+        errorMessage = strprintf(_("Invalid update host: %s"),
+                                 mUpdateHost.c_str());
+        mState = STATE_ERROR;
+        return;
     }
 
+    mUpdateHost = updateHost;
+    mUpdatesDir = "updates/" + mUpdateHost;
+
     ResourceManager *resman = ResourceManager::getInstance();
 
     // Verify that the updates directory exists. Create if necessary.
@@ -1257,7 +1307,7 @@ void Client::initUpdatesDir()
     {
         if (!resman->mkdir("/" + mUpdatesDir))
         {
-#if defined WIN32
+#if defined _WIN32
             std::string newDir = mLocalDataDir + "\\" + mUpdatesDir;
             std::string::size_type loc = newDir.find("/", 0);
 
@@ -1272,13 +1322,17 @@ void Client::initUpdatesDir()
             {
                 logger->log("Error: %s can't be made, but doesn't exist!",
                             newDir.c_str());
-                errorMessage = _("Error creating updates directory!");
+                errorMessage =
+                    strprintf(_("Error creating updates directory!\n(%s)"),
+                                newDir.c_str());
                 mState = STATE_ERROR;
             }
 #else
             logger->log("Error: %s/%s can't be made, but doesn't exist!",
                         mLocalDataDir.c_str(), mUpdatesDir.c_str());
-            errorMessage = _("Error creating updates directory!");
+            errorMessage =
+                strprintf(_("Error creating updates directory!\n(%s/%s)"),
+                            mLocalDataDir.c_str(), mUpdatesDir.c_str());
             mState = STATE_ERROR;
 #endif
         }
@@ -1293,7 +1347,7 @@ void Client::initScreenshotDir()
     }
     else if (mScreenshotDir.empty())
     {
-#ifdef WIN32
+#ifdef _WIN32
         mScreenshotDir = getSpecialFolderLocation(CSIDL_MYPICTURES);
         if (mScreenshotDir.empty())
             mScreenshotDir = getSpecialFolderLocation(CSIDL_DESKTOP);
@@ -1301,7 +1355,7 @@ void Client::initScreenshotDir()
         mScreenshotDir = std::string(PHYSFS_getUserDir()) + "Desktop";
 #endif
 
-        if (config.getValue("useScreenshotDirectorySuffix", true))
+        if (config.getBoolValue("useScreenshotDirectorySuffix"))
         {
             std::string configScreenshotSuffix =
                 config.getValue("screenshotDirectorySuffix",
@@ -1336,3 +1390,42 @@ void Client::accountLogin(LoginData *loginData)
         config.setValue("username", loginData->username);
     config.setValue("remember", loginData->remember);
 }
+
+void Client::handleVideoResize(int width, int height)
+{
+    // Keep a minimum size. This isn't adhered to by the actual window, but
+    // it keeps some window positions from getting messed up.
+    width = std::max(640, width);
+    height = std::max(480, height);
+
+    if (graphics->getWidth() == width && graphics->getHeight() == height)
+        return;
+
+    if (graphics->changeVideoMode(width,
+                                  height,
+                                  graphics->getBpp(),
+                                  false,
+                                  graphics->getHWAccel()))
+    {
+        videoResized(width, height);
+
+        // Since everything appears to have worked out, remember to store the
+        // new size in the configuration.
+        config.setValue("screenwidth", width);
+        config.setValue("screenheight", height);
+    }
+}
+
+void Client::videoResized(int width, int height)
+{
+    gui->videoResized();
+
+    if (mDesktop)
+        mDesktop->setSize(width, height);
+
+    if (mSetupButton)
+        mSetupButton->setPosition(width - mSetupButton->getWidth() - 3, 3);
+
+    if (mGame)
+        mGame->videoResized(width, height);
+}
