@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2008-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -21,15 +21,13 @@
 
 #include "commandhandler.h"
 
-#include "beingmanager.h"
+#include "actorspritemanager.h"
 #include "channelmanager.h"
 #include "channel.h"
 #include "game.h"
 #include "localplayer.h"
 #include "playerrelations.h"
 
-#include "gui/chat.h"
-
 #include "gui/widgets/channeltab.h"
 #include "gui/widgets/chattab.h"
 
@@ -49,7 +47,8 @@ void CommandHandler::handleCommand(const std::string &command, ChatTab *tab)
 {
     std::string::size_type pos = command.find(' ');
     std::string type(command, 0, pos);
-    std::string args(command, pos == std::string::npos ? command.size() : pos + 1);
+    std::string args(command, pos == std::string::npos ?
+                                     command.size() : pos + 1);
     trim(args);
 
     if (type == "help") // Do help before tabs so they can't override it
@@ -124,14 +123,14 @@ void CommandHandler::handleCommand(const std::string &command, ChatTab *tab)
     {
         handlePresent(args, tab);
     }
-    else if (type == "away")
-    {
-        handleAway(args, tab);
-    }
     else if (type == "showip" && Net::getNetworkType() == ServerInfo::TMWATHENA)
     {
         handleShowIp(args, tab);
     }
+    else if (type == "away")
+    {
+        handleAway(args, tab);
+    }
     else
     {
         tab->chatLog(_("Unknown command."));
@@ -340,8 +339,8 @@ void CommandHandler::handleWhere(const std::string &args, ChatTab *tab)
 {
     std::ostringstream where;
     where << Game::instance()->getCurrentMapName() << ", coordinates: "
-          << ((player_node->getPixelX() - 16) / 32) << ", "
-          << ((player_node->getPixelY() - 32) / 32);
+          << local_player->getTileX() << ", "
+          << local_player->getTileY();
 
     tab->chatLog(where.str(), BY_SERVER);
 }
@@ -386,7 +385,7 @@ void CommandHandler::handleMsg(const std::string &args, ChatTab *tab)
 
     if (msg.length() > 0)
     {
-        std::string playerName = player_node->getName();
+        std::string playerName = local_player->getName();
         std::string tempNick = recvnick;
 
         toLower(playerName);
@@ -395,7 +394,7 @@ void CommandHandler::handleMsg(const std::string &args, ChatTab *tab)
         if (tempNick.compare(playerName) == 0 || args.empty())
             return;
 
-        chatWindow->whisper(recvnick, msg, true);
+        chatWindow->whisper(recvnick, msg, BY_PLAYER);
     }
     else
         tab->chatLog(_("Cannot send empty whispers!"), BY_SERVER);
@@ -502,7 +501,7 @@ void CommandHandler::handleShowIp(const std::string &args, ChatTab *tab)
 {
     if (args.empty())
     {
-        tab->chatLog(player_node->getShowIp() ?
+        tab->chatLog(local_player->getShowIp() ?
                 _("Show IP: On") : _("Show IP: Off"));
         return;
     }
@@ -513,18 +512,18 @@ void CommandHandler::handleShowIp(const std::string &args, ChatTab *tab)
     {
         case 0:
             tab->chatLog(_("Show IP: Off"));
-            player_node->setShowIp(false);
+            local_player->setShowIp(false);
             break;
         case 1:
             tab->chatLog(_("Show IP: On"));
-            player_node->setShowIp(true);
+            local_player->setShowIp(true);
             break;
         case -1:
             tab->chatLog(strprintf(BOOLEAN_OPTIONS, "showip"));
             return;
     }
 
-    beingManager->updatePlayerNames();
+    actorSpriteManager->updatePlayerNames();
 }
 
 void CommandHandler::handlePresent(const std::string &args, ChatTab *tab)
@@ -578,5 +577,5 @@ void CommandHandler::handleUnignore(const std::string &args, ChatTab *tab)
 
 void CommandHandler::handleAway(const std::string &args, ChatTab *tab)
 {
-    player_node->setAway(args);
+    local_player->setAway(args);
 }
