@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -21,14 +21,15 @@
 
 #include "game.h"
 
-#include "beingmanager.h"
+#include "actorspritemanager.h"
+#include "actorsprite.h"
 #include "channelmanager.h"
 #include "client.h"
 #include "commandhandler.h"
 #include "configuration.h"
 #include "effectmanager.h"
+#include "event.h"
 #include "emoteshortcut.h"
-#include "flooritemmanager.h"
 #include "graphics.h"
 #include "itemshortcut.h"
 #include "joystick.h"
@@ -36,23 +37,21 @@
 #include "localplayer.h"
 #include "log.h"
 #include "map.h"
-#include "npc.h"
 #include "particle.h"
 #include "playerrelations.h"
 #include "sound.h"
 
-#include "gui/chat.h"
+#include "gui/chatwindow.h"
 #include "gui/confirmdialog.h"
 #include "gui/debugwindow.h"
 #include "gui/equipmentwindow.h"
 #include "gui/gui.h"
-#include "gui/help.h"
+#include "gui/helpwindow.h"
 #include "gui/inventorywindow.h"
 #include "gui/shortcutwindow.h"
 #include "gui/minimap.h"
-#include "gui/ministatus.h"
+#include "gui/ministatuswindow.h"
 #include "gui/npcdialog.h"
-#include "gui/npcpostdialog.h"
 #include "gui/okdialog.h"
 #include "gui/outfitwindow.h"
 #include "gui/quitdialog.h"
@@ -63,9 +62,8 @@
 #include "gui/skilldialog.h"
 #include "gui/statuswindow.h"
 #include "gui/textdialog.h"
-#include "gui/trade.h"
+#include "gui/tradewindow.h"
 #include "gui/viewport.h"
-#include "gui/windowmenu.h"
 
 #include "gui/widgets/chattab.h"
 #include "gui/widgets/emoteshortcutcontainer.h"
@@ -97,7 +95,6 @@ Joystick *joystick = NULL;
 OkDialog *weightNotice = NULL;
 OkDialog *deathNotice = NULL;
 QuitDialog *quitDialog = NULL;
-OkDialog *disconnectedDialog = NULL;
 
 ChatWindow *chatWindow;
 StatusWindow *statusWindow;
@@ -115,8 +112,7 @@ OutfitWindow *outfitWindow;
 SpecialsWindow *specialsWindow;
 SocialWindow *socialWindow;
 
-BeingManager *beingManager = NULL;
-FloorItemManager *floorItemManager = NULL;
+ActorSpriteManager *actorSpriteManager = NULL;
 ChannelManager *channelManager = NULL;
 CommandHandler *commandHandler = NULL;
 Particle *particleEngine = NULL;
@@ -130,14 +126,17 @@ ChatTab *localChatTab = NULL;
  */
 static void initEngines()
 {
-    beingManager = new BeingManager;
+    Event::trigger(Event::GameChannel, Event::EnginesInitializing);
+
+    actorSpriteManager = new ActorSpriteManager;
     commandHandler = new CommandHandler;
-    floorItemManager = new FloorItemManager;
     channelManager = new ChannelManager;
     effectManager = new EffectManager;
 
     particleEngine = new Particle(NULL);
     particleEngine->setupEngine();
+
+    Event::trigger(Event::GameChannel, Event::EnginesInitialized);
 }
 
 /**
@@ -145,6 +144,8 @@ static void initEngines()
  */
 static void createGuiWindows()
 {
+    Event::trigger(Event::GameChannel, Event::GuiWindowsLoading);
+
     setupWindow->clearWindowsForReset();
 
     // Create dialogs
@@ -152,9 +153,9 @@ static void createGuiWindows()
     minimap = new Minimap;
     chatWindow = new ChatWindow;
     tradeWindow = new TradeWindow;
-    equipmentWindow = new EquipmentWindow(player_node->mEquipment.get());
+    equipmentWindow = new EquipmentWindow(PlayerInfo::getEquipment());
     statusWindow = new StatusWindow;
-    inventoryWindow = new InventoryWindow(player_node->getInventory());
+    inventoryWindow = new InventoryWindow(PlayerInfo::getInventory());
     skillDialog = new SkillDialog;
     helpWindow = new HelpWindow;
     debugWindow = new DebugWindow;
@@ -168,12 +169,9 @@ static void createGuiWindows()
 
     localChatTab = new ChatTab(_("General"));
 
-    if (config.getValue("logToChat", 0))
-    {
-        logger->setChatWindow(chatWindow);
-    }
+    NpcDialog::setup();
 
-    Net::getGeneralHandler()->guiWindowsLoaded();
+    Event::trigger(Event::GameChannel, Event::GuiWindowsLoaded);
 }
 
 #define del_0(X) { delete X; X = 0; }
@@ -183,8 +181,8 @@ static void createGuiWindows()
  */
 static void destroyGuiWindows()
 {
-    Net::getGeneralHandler()->guiWindowsUnloaded();
-    logger->setChatWindow(NULL);
+    Event::trigger(Event::GameChannel, Event::GuiWindowsUnloading);
+
     del_0(localChatTab) // Need to do this first, so it can remove itself
     del_0(chatWindow)
     del_0(statusWindow)
@@ -201,23 +199,25 @@ static void destroyGuiWindows()
     del_0(outfitWindow)
     del_0(specialsWindow)
     del_0(socialWindow)
+
+    Event::trigger(Event::NpcChannel, Event::CloseAll); // Cleanup remaining NPC dialogs
+
+    Event::trigger(Event::GameChannel, Event::GuiWindowsUnloaded);
 }
 
 Game *Game::mInstance = 0;
 
 Game::Game():
-    mLastTarget(Being::UNKNOWN),
-    mCurrentMap(0), mMapName("")
+    mLastTarget(ActorSprite::UNKNOWN),
+    mDisconnected(false),
+    mCurrentMap(0)
 {
     assert(!mInstance);
     mInstance = this;
 
-    disconnectedDialog = NULL;
-
     // Create the viewport
     viewport = new Viewport;
-    viewport->setDimension(gcn::Rectangle(0, 0, graphics->getWidth(),
-                                          graphics->getHeight()));
+    viewport->setSize(graphics->getWidth(), graphics->getHeight());
 
     gcn::Container *top = static_cast<gcn::Container*>(gui->getTop());
     top->add(viewport);
@@ -230,21 +230,8 @@ Game::Game():
 
     initEngines();
 
-    Net::getGameHandler()->inGame();
-
     // Initialize beings
-    beingManager->setPlayer(player_node);
-
-    /*
-     * To prevent the server from sending data before the client
-     * has initialized, I've modified it to wait for a "ping"
-     * from the client to complete its initialization
-     *
-     * Note: This only affects the latest eAthena version.  This
-     * packet is handled by the older version, but its response
-     * is ignored by the client
-     */
-    Net::getGameHandler()->ping(tick_time);
+    actorSpriteManager->setPlayer(local_player);
 
     Joystick::init();
     // TODO: The user should be able to choose which one to use
@@ -253,18 +240,21 @@ Game::Game():
         joystick = new Joystick(0);
 
     setupWindow->setInGame(true);
+
+    Event::trigger(Event::GameChannel, Event::Constructed);
 }
 
 Game::~Game()
 {
+    Event::trigger(Event::GameChannel, Event::Destructing);
+
     delete mWindowMenu;
 
     destroyGuiWindows();
 
-    del_0(beingManager)
+    del_0(actorSpriteManager)
     if (Client::getState() != STATE_CHANGE_MAP)
-        del_0(player_node)
-    del_0(floorItemManager)
+        del_0(local_player)
     del_0(channelManager)
     del_0(commandHandler)
     del_0(joystick)
@@ -273,6 +263,8 @@ Game::~Game()
     del_0(mCurrentMap)
 
     mInstance = 0;
+
+    Event::trigger(Event::GameChannel, Event::Destructed);
 }
 
 static bool saveScreenshot()
@@ -280,11 +272,11 @@ static bool saveScreenshot()
     static unsigned int screenshotCount = 0;
 
     // We don't want to show IP addresses in screenshots
-    const bool showip = player_node->getShowIp();
+    const bool showip = local_player->getShowIp();
     if (showip)
     {
-        player_node->setShowIp(false);
-        beingManager->updatePlayerNames();
+        local_player->setShowIp(false);
+        actorSpriteManager->updatePlayerNames();
         gui->draw();
     }
 
@@ -292,8 +284,8 @@ static bool saveScreenshot()
 
     if (showip)
     {
-        player_node->setShowIp(true);
-        beingManager->updatePlayerNames();
+        local_player->setShowIp(true);
+        actorSpriteManager->updatePlayerNames();
     }
 
     // Search for an unused screenshot name
@@ -311,7 +303,8 @@ static bool saveScreenshot()
         screenshotDirectory = std::string(PHYSFS_getUserDir());
     }
 
-    do {
+    do
+    {
         screenshotCount++;
         filenameSuffix.str("");
         filename.str("");
@@ -322,7 +315,8 @@ static bool saveScreenshot()
         testExists.open(filename.str().c_str(), std::ios::in);
         found = !testExists.is_open();
         testExists.close();
-    } while (!found);
+    }
+    while (!found);
 
     const bool success = ImageWriter::writePNG(screenshot, filename.str());
 
@@ -331,11 +325,11 @@ static bool saveScreenshot()
         std::stringstream chatlogentry;
         // TODO: Make it one complete gettext string below
         chatlogentry << _("Screenshot saved as ") << filenameSuffix.str();
-        localChatTab->chatLog(chatlogentry.str(), BY_SERVER);
+        SERVER_NOTICE(chatlogentry.str())
     }
     else
     {
-        localChatTab->chatLog(_("Saving screenshot failed!"), BY_SERVER);
+        SERVER_NOTICE(_("Saving screenshot failed!"))
         logger->log("Error: could not save screenshot.");
     }
 
@@ -349,7 +343,8 @@ void Game::logic()
     handleInput();
 
     // Handle all necessary game logic
-    beingManager->logic();
+    ActorSprite::actorLogic();
+    actorSpriteManager->logic();
     particleEngine->update();
     if (mCurrentMap)
         mCurrentMap->update();
@@ -357,7 +352,7 @@ void Game::logic()
     cur_time = time(NULL);
 
     // Handle network stuff
-    if (!Net::getGameHandler()->isConnected())
+    if (!Net::getGameHandler()->isConnected() && !mDisconnected)
     {
         if (Client::getState() == STATE_CHANGE_MAP)
             return; // Not a problem here
@@ -366,17 +361,29 @@ void Game::logic()
             return; // Disconnect gets handled by STATE_ERROR
 
         errorMessage = _("The connection to the server was lost.");
-
-        if (!disconnectedDialog)
-        {
-            disconnectedDialog = new OkDialog(_("Network Error"),
-                                              errorMessage);
-            disconnectedDialog->addActionListener(&errorListener);
-            disconnectedDialog->requestMoveToTop();
-        }
+        Client::instance()->showOkDialog(_("Network Error"),
+                                         errorMessage,
+                                         STATE_CHOOSE_SERVER);
+        mDisconnected = true;
     }
 }
 
+/**
+ * handle item pick up case.
+ */
+static void handleItemPickUp()
+{
+    int x = local_player->getTileX();
+    int y = local_player->getTileY();
+
+    // Let's look for items around until you find one.
+    FloorItem *item = actorSpriteManager->findItem(x, y, 1);
+    if (!item)
+        return;
+
+    local_player->pickUp(item);
+}
+
 /**
  * The huge input handling method.
  */
@@ -391,8 +398,13 @@ void Game::handleInput()
     {
         bool used = false;
 
+        if (event.type == SDL_VIDEORESIZE)
+        {
+            // Let the client deal with this one (it'll pass down from there)
+            Client::instance()->handleVideoResize(event.resize.w, event.resize.h);
+        }
         // Keyboard events (for discontinuous keys)
-        if (event.type == SDL_KEYDOWN)
+        else if (event.type == SDL_KEYDOWN)
         {
             gcn::Window *requestedWindow = NULL;
 
@@ -407,7 +419,7 @@ void Game::handleInput()
 
             // send straight to gui for certain windows
             if (quitDialog || TextDialog::isActive() ||
-                NpcPostDialog::isActive())
+                    PlayerInfo::getNPCPostCount() > 0)
             {
                 try
                 {
@@ -465,7 +477,8 @@ void Game::handleInput()
             }
 
 
-            if (!chatWindow->isInputFocused() || (event.key.keysym.mod & KMOD_ALT))
+            if (!chatWindow->isInputFocused() || (event.key.keysym.mod &
+                                                  KMOD_ALT))
             {
                 if (keyboard.isKeyActive(keyboard.KEY_PREV_CHAT_TAB))
                 {
@@ -574,16 +587,26 @@ void Game::handleInput()
                     }
                     used = true;
                     break;
-               // Quitting confirmation dialog
-               case KeyboardConfig::KEY_QUIT:
+                case KeyboardConfig::KEY_QUIT:
+                {
+                    // Close possible stuck NPC dialogs.
+                    NpcDialog *npcDialog = NpcDialog::getActive();
+                    if (npcDialog && npcDialog->isWaitingForTheServer())
+                    {
+                        npcDialog->close();
+                        return;
+                    }
+
+                    // Otherwise, show the quit confirmation dialog.
                     quitDialog = new QuitDialog(&quitDialog);
                     quitDialog->requestMoveToTop();
                     return;
+                }
                 default:
                     break;
             }
             if (keyboard.isEnabled() && !chatWindow->isInputFocused() &&
-                !NpcDialog::isAnyInputFocused())
+                !NpcDialog::isAnyInputFocused() && !InventoryWindow::isAnyInputFocused())
             {
                 const int tKey = keyboard.getKeyIndex(event.key.keysym.sym);
 
@@ -608,40 +631,14 @@ void Game::handleInput()
                 {
                     case KeyboardConfig::KEY_PICKUP:
                         {
-                            int x = player_node->getTileX();
-                            int y = player_node->getTileY();
-
-                            FloorItem *item =
-                                floorItemManager->findByCoordinates(x, y);
-
-                            // If none below the player, try the tile in front
-                            // of the player
-                            if (!item)
-                            {
-                                // Temporary until tile-based picking is
-                                // removed.
-                                switch (player_node->getSpriteDirection())
-                                {
-                                    case DIRECTION_UP   : --y; break;
-                                    case DIRECTION_DOWN : ++y; break;
-                                    case DIRECTION_LEFT : --x; break;
-                                    case DIRECTION_RIGHT: ++x; break;
-                                    default: break;
-                                }
-
-                                item = floorItemManager->findByCoordinates(
-                                        x, y);
-                            }
-
-                            if (item)
-                                player_node->pickUp(item);
+                            handleItemPickUp();
 
                             used = true;
                         }
                         break;
                     case KeyboardConfig::KEY_SIT:
                         // Player sit action
-                        player_node->toggleSit();
+                        local_player->toggleSit();
                         used = true;
                         break;
                     case KeyboardConfig::KEY_HIDE_WINDOWS:
@@ -699,26 +696,17 @@ void Game::handleInput()
                         saveScreenshot();
                         used = true;
                         break;
-                    case KeyboardConfig::KEY_PATHFIND:
-                        // Find path to mouse (debug purpose)
-                        viewport->toggleDebugPath();
-                        used = true;
-                        break;
                     case KeyboardConfig::KEY_TRADE:
                         // Toggle accepting of incoming trade requests
                         unsigned int deflt = player_relations.getDefault();
                         if (deflt & PlayerRelation::TRADE)
                         {
-                            localChatTab->chatLog(
-                                        _("Ignoring incoming trade requests"),
-                                        BY_SERVER);
+                            SERVER_NOTICE(_("Ignoring incoming trade requests"))
                             deflt &= ~PlayerRelation::TRADE;
                         }
                         else
                         {
-                            localChatTab->chatLog(
-                                        _("Accepting incoming trade requests"),
-                                        BY_SERVER);
+                            SERVER_NOTICE(_("Accepting incoming trade requests"))
                             deflt |= PlayerRelation::TRADE;
                         }
 
@@ -764,7 +752,7 @@ void Game::handleInput()
         return;
 
     // Moving player around
-    if (player_node->isAlive() && !NPC::isTalking() &&
+    if (local_player->isAlive() && !PlayerInfo::isTalking() &&
         !chatWindow->isInputFocused() && !quitDialog && !TextDialog::isActive())
     {
         // Get the state of the keyboard keys
@@ -806,39 +794,39 @@ void Game::handleInput()
 
         if (keyboard.isKeyActive(keyboard.KEY_EMOTE) && direction != 0)
         {
-            if (player_node->getDirection() != direction)
+            if (local_player->getDirection() != direction)
             {
-                player_node->setDirection(direction);
+                local_player->setDirection(direction);
                 Net::getPlayerHandler()->setDirection(direction);
             }
             direction = 0;
         }
         else
         {
-            player_node->setWalkingDir(direction);
+            local_player->setWalkingDir(direction);
         }
 
         // Attacking monsters
         if (keyboard.isKeyActive(keyboard.KEY_ATTACK) ||
            (joystick && joystick->buttonPressed(0)))
         {
-            if (player_node->getTarget())
-                player_node->attack(player_node->getTarget(), true);
+            if (local_player->getTarget())
+                local_player->attack(local_player->getTarget(), true);
         }
 
         if (keyboard.isKeyActive(keyboard.KEY_TARGET_ATTACK))
         {
-            Being *target = 0;
+            Being *target = local_player->getTarget();
 
             bool newTarget = !keyboard.isKeyActive(keyboard.KEY_TARGET);
             // A set target has highest priority
-            if (!player_node->getTarget())
+            if (!target)
             {
                 // Only auto target Monsters
-                target = beingManager->findNearestLivingBeing(player_node,
-                         20, Being::MONSTER);
+                target = actorSpriteManager->findNearestLivingBeing(local_player,
+                                                    20, ActorSprite::MONSTER);
             }
-            player_node->attack(target, newTarget);
+            local_player->attack(target, newTarget);
         }
 
         // Target the nearest player/monster/npc
@@ -848,40 +836,40 @@ void Game::handleInput()
                     (joystick && joystick->buttonPressed(3))) &&
                 !keyboard.isKeyActive(keyboard.KEY_TARGET))
         {
-            Being::Type currentTarget = Being::UNKNOWN;
+            ActorSprite::Type currentTarget = ActorSprite::UNKNOWN;
             if (keyboard.isKeyActive(keyboard.KEY_TARGET_CLOSEST) ||
                     (joystick && joystick->buttonPressed(3)))
-                currentTarget = Being::MONSTER;
+                currentTarget = ActorSprite::MONSTER;
             else if (keyboard.isKeyActive(keyboard.KEY_TARGET_PLAYER))
-                currentTarget = Being::PLAYER;
+                currentTarget = ActorSprite::PLAYER;
             else if (keyboard.isKeyActive(keyboard.KEY_TARGET_NPC))
-                currentTarget = Being::NPC;
+                currentTarget = ActorSprite::NPC;
 
-            Being *target = beingManager->findNearestLivingBeing(player_node,
+            Being *target = actorSpriteManager->findNearestLivingBeing(local_player,
                                                     20, currentTarget);
 
-            if (target && (target != player_node->getTarget() ||
+            if (target && (target != local_player->getTarget() ||
                     currentTarget != mLastTarget))
             {
-                player_node->setTarget(target);
+                local_player->setTarget(target);
                 mLastTarget = currentTarget;
             }
         }
         else
         {
-            mLastTarget = Being::UNKNOWN; // Reset last target
+            mLastTarget = ActorSprite::UNKNOWN; // Reset last target
         }
 
         // Talk to the nearest NPC if 't' pressed
         if (event.type == SDL_KEYDOWN &&
             keyboard.getKeyIndex(event.key.keysym.sym) == KeyboardConfig::KEY_TALK)
         {
-            Being *target = player_node->getTarget();
+            Being *target = local_player->getTarget();
 
             if (target)
             {
-                if (target->getType() == Being::NPC)
-                    static_cast<NPC*>(target)->talk();
+                if (target->canTalk())
+                    target->talkTo();
             }
         }
 
@@ -889,24 +877,24 @@ void Game::handleInput()
         if (!keyboard.isKeyActive(keyboard.KEY_ATTACK)
             && keyboard.isKeyActive(keyboard.KEY_TARGET))
         {
-            player_node->stopAttack();
+            local_player->stopAttack();
         }
 
         if (joystick)
         {
             if (joystick->buttonPressed(1))
             {
-                const int x = player_node->getTileX();
-                const int y = player_node->getTileY();
+                const int x = local_player->getTileX();
+                const int y = local_player->getTileY();
 
-                FloorItem *item = floorItemManager->findByCoordinates(x, y);
+                FloorItem *item = actorSpriteManager->findItem(x, y);
 
                 if (item)
-                    player_node->pickUp(item);
+                    local_player->pickUp(item);
             }
             else if (joystick->buttonPressed(2))
             {
-                player_node->toggleSit();
+                local_player->toggleSit();
             }
         }
     }
@@ -919,8 +907,7 @@ void Game::handleInput()
 void Game::changeMap(const std::string &mapPath)
 {
     // Clean up floor items, beings and particles
-    floorItemManager->clear();
-    beingManager->clear();
+    actorSpriteManager->clear();
 
     // Close the popup menu on map change so that invalid options can't be
     // executed.
@@ -928,8 +915,8 @@ void Game::changeMap(const std::string &mapPath)
 
     // Unset the map of the player so that its particles are cleared before
     // being deleted in the next step
-    if (player_node)
-        player_node->setMap(0);
+    if (local_player)
+        local_player->setMap(0);
 
     particleEngine->clear();
 
@@ -953,7 +940,7 @@ void Game::changeMap(const std::string &mapPath)
 
     // Notify the minimap and beingManager about the map change
     minimap->setMap(newMap);
-    beingManager->setMap(newMap);
+    actorSpriteManager->setMap(newMap);
     particleEngine->setMap(newMap);
     viewport->setMap(newMap);
 
@@ -967,13 +954,37 @@ void Game::changeMap(const std::string &mapPath)
     if (newMusic != oldMusic)
     {
         if (newMusic.empty())
-            sound.stopMusic();
+            sound.fadeOutMusic();
         else
-            sound.playMusic(newMusic);
+            sound.fadeOutAndPlayMusic(newMusic);
     }
 
     delete mCurrentMap;
     mCurrentMap = newMap;
 
-    Net::getGameHandler()->mapLoaded(mapPath);
+    Event event(Event::MapLoaded);
+    event.setString("mapPath", mapPath);
+    event.trigger(Event::GameChannel);
+}
+
+int Game::getCurrentTileWidth() const
+{
+    if (mCurrentMap)
+        return mCurrentMap->getTileWidth();
+
+    return DEFAULT_TILE_LENGTH;
+}
+
+int Game::getCurrentTileHeight() const
+{
+    if (mCurrentMap)
+        return mCurrentMap->getTileHeight();
+
+    return DEFAULT_TILE_LENGTH;
+}
+
+void Game::videoResized(int width, int height)
+{
+    viewport->setSize(width, height);
+    mWindowMenu->setPosition(width - 3 - mWindowMenu->getWidth(), 3);
 }
