@@ -1,6 +1,6 @@
 /*
  *  The Mana Client
- *  Copyright (C) 2010  The Mana Developers
+ *  Copyright (C) 2010-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -23,8 +23,8 @@
 
 #include "gui/widgets/popup.h"
 
+class Being;
 class Label;
-class Player;
 
 /**
  * A popup that displays information about a being.
@@ -32,20 +32,14 @@ class Player;
 class BeingPopup : public Popup
 {
     public:
-        /**
-         * Constructor. Initializes the being popup.
-         */
         BeingPopup();
 
-        /**
-         * Destructor. Cleans up the being popup on deletion.
-         */
         ~BeingPopup();
 
         /**
          * Sets the info to be displayed given a particular player.
          */
-        void show(int x, int y, Player *p);
+        void show(int x, int y, Being *b);
 
         // TODO: Add a version for monsters, NPCs, etc?
 
