@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -24,8 +24,6 @@
 
 #include "gui/widgets/window.h"
 
-class Label;
-
 /**
  * The debug window.
  *
@@ -34,23 +32,7 @@ class Label;
 class DebugWindow : public Window
 {
     public:
-        /**
-         * Constructor.
-         */
         DebugWindow();
-
-        /**
-         * Logic (updates components' size and infos)
-         */
-        void logic();
-
-    private:
-        Label *mMusicFileLabel, *mMapLabel, *mMinimapLabel;
-        Label *mTileMouseLabel, *mFPSLabel;
-        Label *mParticleCountLabel, *mParticleDetailLabel;
-        Label *mAmbientDetailLabel;
-
-        std::string mFPSText;
 };
 
 extern DebugWindow *debugWindow;
