@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -36,11 +36,6 @@ class TextBox;
 class OkDialog : public Window, public gcn::ActionListener
 {
     public:
-        /**
-         * Constructor.
-         *
-         * @see Window::Window
-         */
         OkDialog(const std::string &title, const std::string &msg,
                  bool modal = true, Window *parent = NULL);
 
@@ -53,4 +48,4 @@ class OkDialog : public Window, public gcn::ActionListener
         TextBox *mTextBox;
 };
 
-#endif
+#endif // OK_DIALOG_H
