@@ -90,13 +90,13 @@ ServersListModel::ServersListModel(ServerInfos *servers, ServerDialog *parent):
 
 int ServersListModel::getNumberOfElements()
 {
-    MutexLocker lock = mParent->lock();
+    MutexLocker lock(mParent->getMutex());
     return mServers->size();
 }
 
 std::string ServersListModel::getElementAt(int elementIndex)
 {
-    MutexLocker lock = mParent->lock();
+    MutexLocker lock(mParent->getMutex());
     const ServerInfo &server = mServers->at(elementIndex);
     std::string myServer;
     myServer += server.hostname;
@@ -282,7 +282,7 @@ void ServerDialog::action(const gcn::ActionEvent &event)
 
         // Check login
         if (index < 0
-#ifndef ENABLE_MANASERV
+#ifndef MANASERV_SUPPORT
             || mServersListModel->getServer(index).type == ServerInfo::MANASERV
 #endif
         )
@@ -475,7 +475,7 @@ void ServerDialog::loadServers()
 
         // Ignore unknown server types
         if (server.type == ServerInfo::UNKNOWN
-#ifndef ENABLE_MANASERV
+#ifndef MANASERV_SUPPORT
             || server.type == ServerInfo::MANASERV
 #endif
         )
@@ -534,6 +534,7 @@ void ServerDialog::loadServers()
                 // Use the name listed in the server list
                 (*it).name = server.name;
                 (*it).version = server.version;
+                (*it).description = server.description;
                 mServersListModel->setVersionString(i, version);
                 found = true;
                 break;
