@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -41,8 +41,8 @@ class Setup_Joystick : public SetupTab, public gcn::ActionListener
     private:
         gcn::Label *mCalibrateLabel;
         gcn::Button *mCalibrateButton;
-        bool mOriginalJoystickEnabled;
-        gcn::CheckBox *mJoystickEnabled;
+        bool mJoystickEnabled;
+        gcn::CheckBox *mJoystickCheckBox;
 };
 
 #endif
