@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -21,11 +21,12 @@
 
 #include "gui/skilldialog.h"
 
-#include "localplayer.h"
 #include "log.h"
+#include "playerinfo.h"
+#include "configuration.h"
+#include "eventlistener.h"
 
 #include "gui/setup.h"
-#include "gui/theme.h"
 
 #include "gui/widgets/button.h"
 #include "gui/widgets/container.h"
@@ -43,6 +44,7 @@
 
 #include "resources/image.h"
 #include "resources/resourcemanager.h"
+#include "resources/theme.h"
 
 #include "utils/dtor.h"
 #include "utils/gettext.h"
@@ -95,7 +97,8 @@ struct SkillInfo
 
         if (!icon)
         {
-            icon = Theme::getImageFromTheme("unknown-item.png");
+            icon = Theme::getImageFromTheme(
+                                       paths.getStringValue("unknownItemFile"));
         }
     }
 
@@ -211,11 +214,15 @@ private:
 SkillDialog::SkillDialog():
     Window(_("Skills"))
 {
+    listen(Event::AttributesChannel);
+
     setWindowName("Skills");
     setCloseButton(true);
     setResizable(true);
     setSaveVisible(true);
     setDefaultSize(windowContainer->getWidth() - 280, 30, 275, 425);
+    setMinHeight(113);
+    setMinWidth(240);
     setupWindow->registerWindowForReset(this);
 
     mTabs = new TabbedArea();
@@ -226,7 +233,7 @@ SkillDialog::SkillDialog():
     place(0, 5, mPointsLabel, 4);
     place(4, 5, mIncreaseButton);
 
-    setLocationRelativeTo(getParent());
+    center();
     loadWindowState();
 }
 
@@ -247,10 +254,6 @@ void SkillDialog::action(const gcn::ActionEvent &event)
     {
         setVisible(false);
     }
-    else
-    {
-        printf("Unknown event '%s'\n", event.getId().c_str());
-    }
 }
 
 std::string SkillDialog::update(int id)
@@ -270,7 +273,7 @@ std::string SkillDialog::update(int id)
 void SkillDialog::update()
 {
     mPointsLabel->setCaption(strprintf(_("Skill points available: %d"),
-                                       player_node->getSkillPoints()));
+                                       PlayerInfo::getAttribute(SKILL_POINTS)));
     mPointsLabel->adjustSize();
 
     for (SkillMap::iterator it = mSkills.begin(); it != mSkills.end(); it++)
@@ -280,6 +283,17 @@ void SkillDialog::update()
     }
 }
 
+void SkillDialog::event(Event::Channel channel, const Event &event)
+{
+    if (event.getType() == Event::UpdateAttribute)
+    {
+        if (event.getInt("id") == SKILL_POINTS)
+        {
+            update();
+        }
+    }
+}
+
 void SkillDialog::clearSkills()
 {
     // Fixes issues with removing tabs
@@ -423,10 +437,10 @@ void SkillModel::updateVisibilities()
 
 void SkillInfo::update()
 {
-    int baseLevel = player_node->getAttributeBase(id);
-    int effLevel = player_node->getAttributeEffective(id);
+    int baseLevel = PlayerInfo::getStatBase(id);
+    int effLevel = PlayerInfo::getStatEffective(id);
 
-    std::pair<int, int> exp = player_node->getExperience(id);
+    std::pair<int, int> exp = PlayerInfo::getStatExperience(id);
 
     if (!modifiable && baseLevel == 0 && effLevel == 0 && exp.second == 0)
     {
