@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -19,12 +19,11 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef SKILL_H
-#define SKILL_H
-
-#include "guichanfwd.h"
+#ifndef SKILLDIALOG_H
+#define SKILLDIALOG_H
 
 #include "gui/widgets/window.h"
+#include "eventlistener.h"
 
 #include <guichan/actionlistener.hpp>
 
@@ -43,13 +42,15 @@ struct SkillInfo;
  *
  * \ingroup Interface
  */
-class SkillDialog : public Window, public gcn::ActionListener
+class SkillDialog : public Window, public gcn::ActionListener, public EventListener
 {
     public:
         SkillDialog();
 
         ~SkillDialog();
 
+        void event(Event::Channel channel, const Event &event);
+
         /**
          * Called when receiving actions from widget.
          */
