@@ -1,6 +1,6 @@
 /*
  *  The Mana Client
- *  Copyright (C) 2010  The Mana Developers
+ *  Copyright (C) 2010-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -20,22 +20,19 @@
 
 #include "gui/socialwindow.h"
 
-#include "beingmanager.h"
+#include "event.h"
 #include "guild.h"
 #include "localplayer.h"
 #include "party.h"
-#include "player.h"
 
 #include "gui/confirmdialog.h"
 #include "gui/okdialog.h"
 #include "gui/setup.h"
 #include "gui/textdialog.h"
-#include "gui/theme.h"
 
 #include "gui/widgets/avatarlistbox.h"
 #include "gui/widgets/browserbox.h"
 #include "gui/widgets/button.h"
-#include "gui/widgets/chattab.h"
 #include "gui/widgets/container.h"
 #include "gui/widgets/label.h"
 #include "gui/widgets/layouthelper.h"
@@ -49,6 +46,8 @@
 #include "net/guildhandler.h"
 #include "net/partyhandler.h"
 
+#include "resources/theme.h"
+
 #include "utils/dtor.h"
 #include "utils/gettext.h"
 #include "utils/stringutils.h"
@@ -124,13 +123,10 @@ public:
 
             if (!name.empty())
             {
-                Net::getGuildHandler()->invite(mGuild->getId(), name);
-                localChatTab->chatLog(strprintf(_("Invited user %s to guild %s."),
-                                            name.c_str(),
-                                            mGuild->getName().c_str()),
-                                  BY_SERVER);
+                SERVER_NOTICE(strprintf(_("Invited user %s to guild %s."),
+	                          name.c_str(),
+	                          mGuild->getName().c_str()))
             }
-
             mInviteDialog = NULL;
         }
         else if (event.getId() == "~do invite")
@@ -140,8 +136,8 @@ public:
         else if (event.getId() == "yes")
         {
             Net::getGuildHandler()->leave(mGuild->getId());
-            localChatTab->chatLog(strprintf(_("Guild %s quit requested."),
-                                        mGuild->getName().c_str()), BY_SERVER);
+            SERVER_NOTICE(strprintf(_("Guild %s quit requested."),
+                                        mGuild->getName().c_str()))
             mConfirmDialog = NULL;
         }
         else if (event.getId() == "no")
@@ -208,8 +204,8 @@ public:
             std::string name = mInviteDialog->getText();
 
             if (!name.empty())
-                Net::getPartyHandler()->invite(name);
-
+                SERVER_NOTICE(strprintf(_("Invited user %s to party."),
+                              name.c_str()))
             mInviteDialog = NULL;
         }
         else if (event.getId() == "~do invite")
@@ -219,8 +215,8 @@ public:
         else if (event.getId() == "yes")
         {
             Net::getPartyHandler()->leave();
-            localChatTab->chatLog(strprintf(_("Party %s quit requested."),
-                                        mParty->getName().c_str()), BY_SERVER);
+            SERVER_NOTICE(strprintf(_("Party %s quit requested."),
+                                        mParty->getName().c_str()))
             mConfirmDialog = NULL;
         }
         else if (event.getId() == "no")
@@ -314,6 +310,7 @@ private:
 
 SocialWindow::SocialWindow() :
     Window(_("Social")),
+    mGuildInvited(0),
     mGuildAcceptDialog(NULL),
     mPartyAcceptDialog(NULL)
 {
@@ -344,9 +341,9 @@ SocialWindow::SocialWindow() :
 
     mCreatePopup = new CreatePopup();
 
-    if (player_node->getParty())
+    if (local_player->getParty())
     {
-        addTab(player_node->getParty());
+        addTab(local_player->getParty());
     }
     else
         updateButtons();
@@ -444,14 +441,14 @@ void SocialWindow::action(const gcn::ActionEvent &event)
         // check if they accepted the invite
         if (eventId == "yes")
         {
-            localChatTab->chatLog(strprintf(_("Accepted party invite from %s."),
-                    mPartyInviter.c_str()));
+            SERVER_NOTICE(strprintf(_("Accepted party invite from %s."),
+                    mPartyInviter.c_str()))
             Net::getPartyHandler()->inviteResponse(mPartyInviter, true);
         }
         else if (eventId == "no")
         {
-            localChatTab->chatLog(strprintf(_("Rejected party invite from %s."),
-                    mPartyInviter.c_str()));
+            SERVER_NOTICE(strprintf(_("Rejected party invite from %s."),
+                    mPartyInviter.c_str()))
             Net::getPartyHandler()->inviteResponse(mPartyInviter, false);
         }
 
@@ -463,14 +460,12 @@ void SocialWindow::action(const gcn::ActionEvent &event)
         // check if they accepted the invite
         if (eventId == "yes")
         {
-            localChatTab->chatLog(strprintf(_("Accepted guild invite from %s."),
-                    mPartyInviter.c_str()));
+            SERVER_NOTICE(_("Accepted guild invite"))
             Net::getGuildHandler()->inviteResponse(mGuildInvited, true);
         }
         else if (eventId == "no")
         {
-            localChatTab->chatLog(strprintf(_("Rejected guild invite from %s."),
-                    mPartyInviter.c_str()));
+            SERVER_NOTICE(_("Rejected guild invite."))
             Net::getGuildHandler()->inviteResponse(mGuildInvited, false);
         }
 
@@ -498,14 +493,15 @@ void SocialWindow::action(const gcn::ActionEvent &event)
 
         if (name.size() > 16)
         {
-            localChatTab->chatLog(_("Creating guild failed, please choose a "
-                                    "shorter name."), BY_SERVER);
+            SERVER_NOTICE(_("Creating guild failed, please choose a "
+                            "shorter name."));
+            return;
         }
         else if (!name.empty())
         {
             Net::getGuildHandler()->create(name);
-            localChatTab->chatLog(strprintf(_("Creating guild called %s."),
-                                                name.c_str()), BY_SERVER);
+            SERVER_NOTICE(strprintf(_("Creating guild called %s."),
+                                    name.c_str()));
         }
 
         mGuildCreateDialog = NULL;
@@ -520,14 +516,15 @@ void SocialWindow::action(const gcn::ActionEvent &event)
 
         if (name.size() > 16)
         {
-            localChatTab->chatLog(_("Creating party failed, please choose a "
-                                    "shorter name."), BY_SERVER);
+            SERVER_NOTICE(_("Creating party failed, please choose a "
+                            "shorter name."));
+            return;
         }
         else if (!name.empty())
         {
             Net::getPartyHandler()->create(name);
-            localChatTab->chatLog(strprintf(_("Creating party called %s."),
-                                                name.c_str()), BY_SERVER);
+            SERVER_NOTICE(strprintf(_("Creating party called %s."),
+                                    name.c_str()));
         }
 
         mPartyCreateDialog = NULL;
@@ -553,14 +550,14 @@ void SocialWindow::showGuildInvite(const std::string &guildName,
     // check there isnt already an invite showing
     if (mGuildInvited != 0)
     {
-        localChatTab->chatLog(_("Received guild request, but one already "
-                "exists."), BY_SERVER);
+        SERVER_NOTICE(_("Received guild request, but one already "
+                "exists."))
         return;
     }
 
     std::string msg = strprintf(_("%s has invited you to join the guild %s."),
                                 inviterName.c_str(), guildName.c_str());
-    localChatTab->chatLog(msg, BY_SERVER);
+    SERVER_NOTICE(msg)
 
     // show invite
     mGuildAcceptDialog = new ConfirmDialog(_("Accept Guild Invite"), msg, this);
@@ -569,14 +566,13 @@ void SocialWindow::showGuildInvite(const std::string &guildName,
     mGuildInvited = guildId;
 }
 
-void SocialWindow::showPartyInvite(const std::string &partyName,
-                                   const std::string &inviter)
+void SocialWindow::showPartyInvite(const std::string &inviter,
+                                   const std::string &partyName)
 {
     // check there isnt already an invite showing
     if (mPartyInviter != "")
     {
-        localChatTab->chatLog(_("Received party request, but one already "
-                "exists."), BY_SERVER);
+        SERVER_NOTICE(_("Received party request, but one already exists."))
         return;
     }
 
@@ -607,7 +603,7 @@ void SocialWindow::showPartyInvite(const std::string &partyName,
         }
     }
 
-    localChatTab->chatLog(msg, BY_SERVER);
+    SERVER_NOTICE(msg)
 
     // show invite
     mPartyAcceptDialog = new ConfirmDialog(_("Accept Party Invite"), msg, this);
@@ -618,7 +614,7 @@ void SocialWindow::showPartyInvite(const std::string &partyName,
 
 void SocialWindow::showPartyCreate()
 {
-    if (player_node->getParty())
+    if (local_player->getParty())
     {
         new OkDialog(_("Create Party"),
                      _("Cannot create party. You are already in a party"),
