@@ -1,6 +1,7 @@
 /*
  *  The Mana Client
- *  Copyright (c) 2009  Aethyra Development Team 
+ *  Copyright (c) 2009  Aethyra Development Team
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -20,19 +21,20 @@
 
 #include "gui/widgets/label.h"
 
-#include "gui/theme.h"
+#include "resources/theme.h"
 
 Label::Label()
 {
+    setForegroundColor(Theme::getThemeColor(Theme::TEXT));
 }
 
 Label::Label(const std::string &caption) :
     gcn::Label(caption)
 {
+    setForegroundColor(Theme::getThemeColor(Theme::TEXT));
 }
 
 void Label::draw(gcn::Graphics *graphics)
 {
-    setForegroundColor(Theme::getThemeColor(Theme::TEXT));
     gcn::Label::draw(static_cast<gcn::Graphics*>(graphics));
 }
