@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -21,6 +21,8 @@
 
 #include "gui/widgets/windowcontainer.h"
 
+#include "gui/widgets/window.h"
+
 #include "utils/dtor.h"
 
 WindowContainer *windowContainer = NULL;
@@ -37,3 +39,11 @@ void WindowContainer::scheduleDelete(gcn::Widget *widget)
 {
     mDeathList.push_back(widget);
 }
+
+void WindowContainer::adjustAfterResize(int oldScreenWidth,
+                                        int oldScreenHeight)
+{
+    for (WidgetListIterator i = mWidgets.begin(); i != mWidgets.end(); ++i)
+        if (Window *window = dynamic_cast<Window*>(*i))
+            window->adjustPositionAfterResize(oldScreenWidth, oldScreenHeight);
+}
