@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -22,6 +22,7 @@
 #ifndef MAP_H
 #define MAP_H
 
+#include "actor.h"
 #include "position.h"
 #include "properties.h"
 
@@ -31,18 +32,16 @@
 class Animation;
 class AmbientLayer;
 class Graphics;
-class Image;
 class MapLayer;
 class Particle;
 class SimpleAnimation;
-class Sprite;
 class Tileset;
 
 typedef std::vector<Tileset*> Tilesets;
-typedef std::list<Sprite*> MapSprites;
-typedef MapSprites::iterator MapSprite;
 typedef std::vector<MapLayer*> Layers;
 
+const int DEFAULT_TILE_LENGTH = 32;
+
 /**
  * A meta tile stores additional information about a location on a tile map.
  * This is information that doesn't need to be repeated for each tile in each
@@ -50,16 +49,13 @@ typedef std::vector<MapLayer*> Layers;
  */
 struct MetaTile
 {
-    /**
-     * Constructor.
-     */
     MetaTile() : whichList(0), blockmask(0) {}
 
     // Pathfinding members
     int Fcost;               /**< Estimation of total path cost */
     int Gcost;               /**< Cost from start to this location */
     int Hcost;               /**< Estimated cost to goal */
-    int whichList;           /**< No list, open list or closed list */
+    unsigned whichList;      /**< No list, open list or closed list */
     int parentX;             /**< X coordinate of parent tile */
     int parentY;             /**< Y coordinate of parent tile */
     unsigned char blockmask; /**< Blocking properties of this tile */
@@ -91,14 +87,12 @@ class MapLayer
     public:
         /**
          * Constructor, taking layer origin, size and whether this layer is the
-         * fringe layer. The fringe layer is the layer that draws the sprites.
+         * fringe layer. The fringe layer is the layer that draws the actors.
          * There can be only one fringe layer per map.
          */
-        MapLayer(int x, int y, int width, int height, bool isFringeLayer);
+        MapLayer(int x, int y, int width, int height, bool isFringeLayer,
+                 Map *map);
 
-        /**
-         * Destructor.
-         */
         ~MapLayer();
 
         /**
@@ -121,21 +115,27 @@ class MapLayer
          * expected to be in map range and will be translated to local layer
          * coordinates and clipped to the layer's dimensions.
          *
-         * The given sprites are only drawn when this layer is the fringe
+         * The given actors are only drawn when this layer is the fringe
          * layer.
          */
         void draw(Graphics *graphics,
                   int startX, int startY,
                   int endX, int endY,
                   int scrollX, int scrollY,
-                  const MapSprites &sprites,
-                  int mDebugFlags) const;
+                  const Actors &actors,
+                  int debugFlags) const;
+
+        bool isFringeLayer()
+        { return mIsFringeLayer; }
+
+        int getTileDrawWidth(int x1, int y1, int endX, int &width) const;
 
     private:
         int mX, mY;
         int mWidth, mHeight;
-        bool mIsFringeLayer;    /**< Whether the sprites are drawn. */
+        bool mIsFringeLayer;    /**< Whether the actors are drawn. */
         Image **mTiles;
+        Map *mMap;              /** The mother map pointer */
 };
 
 /**
@@ -160,11 +160,18 @@ class Map : public Properties
             BLOCKMASK_MONSTER   = 0x02  // = bin 0000 0010
         };
 
-        enum DebugType
+        enum DebugFlags
         {
-            MAP_NORMAL  = 0,
-            MAP_DEBUG   = 1,
-            MAP_SPECIAL = 2
+            DEBUG_NONE = 0x0,
+            DEBUG_GRID = 0x1,
+            DEBUG_COLLISION_TILES = 0x2,
+            DEBUG_BEING_COLLISION_RADIUS = 0x4,
+            DEBUG_BEING_POSITION = 0x8,
+            DEBUG_BEING_PATH = 0x10,
+            DEBUG_MOUSE_PATH = 0x20,
+            DEBUG_SPECIAL1 = 0x40,
+            DEBUG_SPECIAL2 = 0x80,
+            DEBUG_SPECIAL3 = 0x100
         };
 
         /**
@@ -172,9 +179,6 @@ class Map : public Properties
          */
         Map(int width, int height, int tileWidth, int tileHeight);
 
-        /**
-         * Destructor.
-         */
         ~Map();
 
         /**
@@ -190,7 +194,7 @@ class Map : public Properties
 
         /**
          * Draws the map to the given graphics output. This method draws all
-         * layers, sprites and overlay effects.
+         * layers, actors and overlay effects.
          *
          * TODO: For efficiency reasons, this method could take into account
          * the clipping rectangle set on the Graphics object. However,
@@ -217,7 +221,7 @@ class Map : public Properties
         /**
          * Finds the tile set that a tile with the given global id is part of.
          */
-        Tileset *getTilesetWithGid(int gid) const;
+        Tileset *getTilesetWithGid(unsigned gid) const;
 
         /**
          * Get tile reference.
@@ -263,6 +267,14 @@ class Map : public Properties
         int getTileHeight() const
         { return mTileHeight; }
 
+        /**
+         * Returns the tile center position in pixel coordinates.
+         *
+         * @param x the horizontal tile position
+         * @param y the vertical tile position
+         */
+        Vector getTileCenter(int x, int y);
+
         const std::string getMusicFile() const;
         const std::string getName() const;
 
@@ -282,28 +294,20 @@ class Map : public Properties
         { return checkNodeOffsets(radius, walkMask, Position(x, y)); }
 
         /**
-         * Find a pixel path from one location to the next.
+         * Find a tile-centered path in pixel coordinates
+         * from one location to the next.
+         */
+        Path findTilePath(int startPixelX, int startPixelY, int endPixelX,
+                          int endPixelY, unsigned char walkMask,
+                          int maxCost = 20);
+
+        /**
+         * Find a pixel path from one location to the next using free offsets.
          */
         Path findPixelPath(int startPixelX, int startPixelY,
                           int destPixelX, int destPixelY,
                           int radius, unsigned char walkmask, int maxCost = 20);
 
-        /**
-         * Find a path from one location to the next.
-         */
-        Path findPath(int startX, int startY, int destX, int destY,
-                      unsigned char walkmask, int maxCost = 20);
-
-        /**
-         * Adds a sprite to the map.
-         */
-        MapSprite addSprite(Sprite *sprite);
-
-        /**
-         * Removes a sprite from the map.
-         */
-        void removeSprite(MapSprite iterator);
-
         /**
          * Adds a particle effect
          */
@@ -320,16 +324,34 @@ class Map : public Properties
         void addAnimation(int gid, TileAnimation *animation)
         { mTileAnimations[gid] = animation; }
 
-        void setDebugFlags(int n) {mDebugFlags = n;}
+        void setDebugFlags(int flags) { mDebugFlags = flags; }
 
-        int getDebugFlags() const {return mDebugFlags;}
+        int getDebugFlags() const { return mDebugFlags; }
 
         /**
          * Gets the tile animation for a specific gid
          */
         TileAnimation *getAnimationForGid(int gid) const;
 
+    protected:
+        friend class Actor;
+
+        /**
+         * Adds an actor to the map.
+         */
+        Actors::iterator addActor(Actor *actor);
+
+        /**
+         * Removes an actor from the map.
+         */
+        void removeActor(Actors::iterator iterator);
+
     private:
+        /**
+         * Find a path from one location to the next in tile coordinates.
+         */
+        Path findPath(int startX, int startY, int destX, int destY,
+                      unsigned char walkmask, int maxCost = 20);
 
         enum LayerType
         {
@@ -356,7 +378,7 @@ class Map : public Properties
         /**
          * Blockmasks for different entities
          */
-        int *mOccupation[NB_BLOCKTYPES];
+        unsigned *mOccupation[NB_BLOCKTYPES];
 
         int mWidth, mHeight;
         int mTileWidth, mTileHeight;
@@ -364,13 +386,13 @@ class Map : public Properties
         MetaTile *mMetaTiles;
         Layers mLayers;
         Tilesets mTilesets;
-        MapSprites mSprites;
+        Actors mActors;
 
         // debug flags
         int mDebugFlags;
 
         // Pathfinding members
-        int mOnClosedList, mOnOpenList;
+        unsigned mOnClosedList, mOnOpenList;
 
         // Overlay data
         std::list<AmbientLayer*> mBackgrounds;
