@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -19,9 +19,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "client.h"
-#include "main.h"
-
 #ifndef GENERALHANDLER_H
 #define GENERALHANDLER_H
 
@@ -40,13 +37,7 @@ class GeneralHandler
 
         virtual void flushNetwork() = 0;
 
-        virtual void guiWindowsLoaded() = 0;
-
-        virtual void guiWindowsUnloaded() = 0;
-
         virtual void clearHandlers() = 0;
-
-        virtual void stateChanged(State oldState, State newState) = 0;
 };
 
 } // namespace Net
