@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -24,13 +24,17 @@
 
 #include "net/adminhandler.h"
 
+#include "net/manaserv/messagehandler.h"
+
 namespace ManaServ {
 
-class AdminHandler : public Net::AdminHandler
+class AdminHandler : public Net::AdminHandler, public MessageHandler
 {
     public:
         AdminHandler();
 
+        void handleMessage(Net::MessageIn &msg);
+
         void announce(const std::string &text);
 
         void localAnnounce(const std::string &text);
