@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -30,6 +30,7 @@
 #include "gui/skilldialog.h"
 #include "gui/specialswindow.h"
 
+#include "net/manaserv/adminhandler.h"
 #include "net/manaserv/beinghandler.h"
 #include "net/manaserv/buysellhandler.h"
 #include "net/manaserv/charhandler.h"
@@ -46,7 +47,7 @@
 #include "net/manaserv/partyhandler.h"
 #include "net/manaserv/playerhandler.h"
 #include "net/manaserv/specialhandler.h"
-#include "net/manaserv/stats.h"
+#include "net/manaserv/attributes.h"
 #include "net/manaserv/tradehandler.h"
 
 #include "utils/gettext.h"
@@ -67,6 +68,7 @@ ServerInfo gameServer;
 ServerInfo chatServer;
 
 GeneralHandler::GeneralHandler():
+        mAdminHandler(new AdminHandler),
         mBeingHandler(new BeingHandler),
         mBuySellHandler(new BuySellHandler),
         mCharHandler(new CharHandler),
@@ -90,10 +92,14 @@ GeneralHandler::GeneralHandler():
     chatServerConnection = getConnection();
 
     generalHandler = this;
+
+    listen(Event::ClientChannel);
+    listen(Event::GameChannel);
 }
 
 void GeneralHandler::load()
 {
+    registerHandler(mAdminHandler.get());
     registerHandler(mBeingHandler.get());
     registerHandler(mBuySellHandler.get());
     registerHandler(mCharHandler.get());
@@ -127,9 +133,9 @@ void GeneralHandler::reload()
     gameServer.clear();
     chatServer.clear();
 
-    Stats::unload();
-    Stats::load();
-    Stats::informItemDB();
+    Attributes::unload();
+    Attributes::load();
+    Attributes::informItemDB();
 }
 
 void GeneralHandler::unload()
@@ -147,7 +153,7 @@ void GeneralHandler::unload()
     delete gameServerConnection;
     delete chatServerConnection;
 
-    Stats::unload();
+    Attributes::unload();
     finalize();
 }
 
@@ -163,38 +169,43 @@ void GeneralHandler::flushNetwork()
     }
 }
 
-void GeneralHandler::guiWindowsLoaded()
-{
-    inventoryWindow->setSplitAllowed(true);
-    skillDialog->loadSkills();
-    specialsWindow->loadSpecials("specials.xml");
-
-    player_node->setExpNeeded(100);
-
-    Stats::informStatusWindow();
-}
-
-void GeneralHandler::guiWindowsUnloaded()
-{
-    // TODO
-}
-
 void GeneralHandler::clearHandlers()
 {
     clearNetworkHandlers();
 }
 
-void GeneralHandler::stateChanged(State oldState, State newState)
+void GeneralHandler::event(Event::Channel channel,
+                           const Event &event)
 {
-    if (newState == STATE_GAME)
+    if (channel == Event::ClientChannel)
     {
-        GameHandler *game = static_cast<GameHandler*>(Net::getGameHandler());
-        game->gameLoading();
+        if (event.getType() == Event::StateChange)
+        {
+            int newState = event.getInt("newState");
+
+            if (newState == STATE_GAME)
+            {
+                GameHandler *game = static_cast<GameHandler*>(Net::getGameHandler());
+                game->gameLoading();
+            }
+        }
+        else if (event.getType() == Event::LoadingDatabases)
+        {
+            Attributes::load();
+            Attributes::informItemDB();
+        }
     }
-    else if (newState == STATE_LOAD_DATA)
+    else if (channel == Event::GameChannel)
     {
-        Stats::load();
-        Stats::informItemDB();
+        if (event.getType() == Event::GuiWindowsLoaded)
+        {
+            inventoryWindow->setSplitAllowed(true);
+            skillDialog->loadSkills();
+
+            PlayerInfo::setAttribute(EXP_NEEDED, 100);
+
+            Attributes::informStatusWindow();
+        }
     }
 }
 
