@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -24,8 +24,6 @@
 
 #include "net/messagein.h"
 
-#include <SDL_types.h>
-
 #include <memory>
 
 namespace Net {
@@ -36,7 +34,7 @@ namespace Net {
 class MessageHandler
 {
     public:
-        const Uint16 *handledMessages;
+        const uint16_t *handledMessages;
 
         virtual void handleMessage(MessageIn &msg) = 0;
 
