@@ -154,7 +154,7 @@ void connectToServer(ServerInfo &server)
 
         switch (server.type)
         {
-#ifdef ENABLE_MANASERV
+#ifdef MANASERV_SUPPORT
             case ServerInfo::MANASERV:
                 new ManaServ::GeneralHandler;
                 break;
