@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -32,6 +32,10 @@
 #include "gui/socialwindow.h"
 #include "gui/statuswindow.h"
 
+#include "net/messagein.h"
+#include "net/messageout.h"
+#include "net/serverinfo.h"
+
 #include "net/tmwa/adminhandler.h"
 #include "net/tmwa/beinghandler.h"
 #include "net/tmwa/buysellhandler.h"
@@ -53,9 +57,6 @@
 #include "net/tmwa/gui/guildtab.h"
 #include "net/tmwa/gui/partytab.h"
 
-#include "net/messagein.h"
-#include "net/messageout.h"
-
 #include "resources/itemdb.h"
 
 #include "utils/gettext.h"
@@ -75,7 +76,7 @@ extern Party *taParty;
 
 GeneralHandler::GeneralHandler():
     mAdminHandler(new AdminHandler),
-    mBeingHandler(new BeingHandler(config.getValue("EnableSync", 0) == 1)),
+    mBeingHandler(new BeingHandler(config.getBoolValue("EnableSync"))),
     mBuySellHandler(new BuySellHandler),
     mCharHandler(new CharServerHandler),
     mChatHandler(new ChatHandler),
@@ -97,15 +98,17 @@ GeneralHandler::GeneralHandler():
     handledMessages = _messages;
     generalHandler = this;
 
-    std::list<ItemDB::Stat> stats;
-    stats.push_back(ItemDB::Stat("str", _("Strength %+d")));
-    stats.push_back(ItemDB::Stat("agi", _("Agility %+d")));
-    stats.push_back(ItemDB::Stat("vit", _("Vitality %+d")));
-    stats.push_back(ItemDB::Stat("int", _("Intelligence %+d")));
-    stats.push_back(ItemDB::Stat("dex", _("Dexterity %+d")));
-    stats.push_back(ItemDB::Stat("luck", _("Luck %+d")));
+    std::list<ItemStat> stats;
+    stats.push_back(ItemStat("str", _("Strength %+d")));
+    stats.push_back(ItemStat("agi", _("Agility %+d")));
+    stats.push_back(ItemStat("vit", _("Vitality %+d")));
+    stats.push_back(ItemStat("int", _("Intelligence %+d")));
+    stats.push_back(ItemStat("dex", _("Dexterity %+d")));
+    stats.push_back(ItemStat("luck", _("Luck %+d")));
 
-    ItemDB::setStatsList(stats);
+    setStatsList(stats);
+
+    listen(Event::GameChannel);
 }
 
 GeneralHandler::~GeneralHandler()
@@ -209,47 +212,54 @@ void GeneralHandler::flushNetwork()
     }
 }
 
-void GeneralHandler::guiWindowsLoaded()
-{
-    inventoryWindow->setSplitAllowed(false);
-    skillDialog->loadSkills();
-
-    statusWindow->addAttribute(STR, _("Strength"), true, "");
-    statusWindow->addAttribute(AGI, _("Agility"), true, "");
-    statusWindow->addAttribute(VIT, _("Vitality"), true, "");
-    statusWindow->addAttribute(INT, _("Intelligence"), true, "");
-    statusWindow->addAttribute(DEX, _("Dexterity"), true, "");
-    statusWindow->addAttribute(LUK, _("Luck"), true, "");
-
-    statusWindow->addAttribute(ATK, _("Attack"), false, "");
-    statusWindow->addAttribute(DEF, _("Defense"), false, "");
-    statusWindow->addAttribute(MATK, _("M.Attack"), false, "");
-    statusWindow->addAttribute(MDEF, _("M.Defense"), false, "");
-    statusWindow->addAttribute(HIT, _("% Accuracy"), false, "");
-    statusWindow->addAttribute(FLEE, _("% Evade"), false, "");
-    statusWindow->addAttribute(CRIT, _("% Critical"), false, "");
-}
-
-void GeneralHandler::guiWindowsUnloaded()
-{
-    socialWindow->removeTab(taGuild);
-    socialWindow->removeTab(taParty);
-
-    delete guildTab;
-    guildTab = 0;
-
-    delete partyTab;
-    partyTab = 0;
-}
-
 void GeneralHandler::clearHandlers()
 {
     mNetwork->clearHandlers();
 }
 
-void GeneralHandler::stateChanged(State oldState, State newState)
+void GeneralHandler::event(Event::Channel channel,
+                           const Event &event)
 {
-    //
+    if (channel == Event::GameChannel)
+    {
+        if (event.getType() == Event::GuiWindowsLoaded)
+        {
+            inventoryWindow->setSplitAllowed(false);
+            skillDialog->loadSkills();
+
+            statusWindow->addAttribute(STRENGTH, _("Strength"), true, "");
+            statusWindow->addAttribute(AGILITY, _("Agility"), true, "");
+            statusWindow->addAttribute(VITALITY, _("Vitality"), true, "");
+            statusWindow->addAttribute(INTELLIGENCE, _("Intelligence"),
+                                       true, "");
+            statusWindow->addAttribute(DEXTERITY, _("Dexterity"), true, "");
+            statusWindow->addAttribute(LUCK, _("Luck"), true, "");
+
+            statusWindow->addAttribute(ATK, _("Attack"), false, "");
+            statusWindow->addAttribute(DEF, _("Defense"), false, "");
+            statusWindow->addAttribute(MATK, _("M.Attack"), false, "");
+            statusWindow->addAttribute(MDEF, _("M.Defense"), false, "");
+            // NOTE: Don't remove the gettext comments as they are used
+            // by the xgettext invocation.
+            //xgettext:no-c-format
+            statusWindow->addAttribute(HIT, _("% Accuracy"), false, "");
+            //xgettext:no-c-format
+            statusWindow->addAttribute(FLEE, _("% Evade"), false, "");
+            //xgettext:no-c-format
+            statusWindow->addAttribute(CRIT, _("% Critical"), false, "");
+        }
+        else if (event.getType() == Event::GuiWindowsUnloading)
+        {
+            socialWindow->removeTab(taGuild);
+            socialWindow->removeTab(taParty);
+
+            delete guildTab;
+            guildTab = 0;
+
+            delete partyTab;
+            partyTab = 0;
+        }
+    }
 }
 
 } // namespace TmwAthena
