@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -24,7 +24,6 @@
 
 #include "net/messagein.h"
 
-#include <SDL_types.h>
 #include <string>
 
 namespace TmwAthena {
@@ -37,13 +36,10 @@ namespace TmwAthena {
     class MessageIn : public Net::MessageIn
 {
     public:
-        /**
-         * Constructor.
-         */
         MessageIn(const char *data, unsigned int length);
 
-        int readInt16();            /**< Reads a short. */
-        int readInt32();            /**< Reads a long. */
+        uint16_t readInt16();
+        uint32_t readInt32();
 };
 
 }
