@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -24,9 +24,6 @@
 
 #include "net/messageout.h"
 
-#include <iosfwd>
-#include <SDL_types.h>
-
 namespace TmwAthena {
 
 class Network;
@@ -39,19 +36,16 @@ class Network;
 class MessageOut : public Net::MessageOut
 {
     public:
-        /**
-         * Constructor.
-         */
-        MessageOut(short id);
+        MessageOut(uint16_t id);
 
-        void writeInt16(Sint16 value);        /**< Writes a short. */
-        void writeInt32(Sint32 value);        /**< Writes a long. */
+        void writeInt16(uint16_t value);
+        void writeInt32(uint32_t value);
 
         /**
          * Encodes coordinates and direction in 3 bytes.
          */
-        void writeCoordinates(unsigned short x, unsigned short y,
-                              unsigned char direction);
+        void writeCoordinates(uint16_t x, uint16_t y,
+                              uint8_t direction);
 
     private:
         void expand(size_t size);
