@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -20,30 +20,28 @@
  */
 
 #include "net/tmwa/playerhandler.h"
+#include "net/tmwa/beinghandler.h"
 
+#include "client.h"
 #include "configuration.h"
 #include "game.h"
 #include "localplayer.h"
 #include "log.h"
-#include "npc.h"
+#include "playerinfo.h"
 #include "units.h"
 
-#include "gui/buy.h"
-#include "gui/buysell.h"
+#include "gui/buydialog.h"
+#include "gui/buyselldialog.h"
 #include "gui/gui.h"
-#include "gui/npcdialog.h"
 #include "gui/okdialog.h"
-#include "gui/sell.h"
+#include "gui/selldialog.h"
 #include "gui/statuswindow.h"
 #include "gui/viewport.h"
 
-#include "gui/widgets/chattab.h"
-
 #include "net/messagein.h"
 #include "net/messageout.h"
 
 #include "net/tmwa/protocol.h"
-#include "net/tmwa/npchandler.h"
 
 #include "utils/stringutils.h"
 #include "utils/gettext.h"
@@ -53,10 +51,7 @@ extern OkDialog *deathNotice;
 
 // Max. distance we are willing to scroll after a teleport;
 // everything beyond will reset the port hard.
-static const int MAP_TELEPORT_SCROLL_DISTANCE = 8;
-
-#define ATTR_BONUS(atr) \
-(player_node->getAttributeEffective(atr) - player_node->getAttributeBase(atr))
+const int MAP_TELEPORT_SCROLL_DISTANCE = 8;
 
 // TODO Move somewhere else
 namespace {
@@ -84,14 +79,11 @@ namespace {
 
             BuyDialog::closeAll();
             BuySellDialog::closeAll();
-            NpcDialog::closeAll();
             SellDialog::closeAll();
 
             viewport->closePopupMenu();
 
-            TmwAthena::NpcHandler *handler =
-                    static_cast<TmwAthena::NpcHandler*>(Net::getNpcHandler());
-            handler->clearDialogs();
+            Event::trigger(Event::NpcChannel, Event::CloseAll);
         }
     } deathListener;
 
@@ -169,7 +161,7 @@ PlayerHandler::PlayerHandler()
 
 void PlayerHandler::handleMessage(Net::MessageIn &msg)
 {
-    if (!player_node)
+    if (!local_player)
         return;
 
     switch (msg.getId())
@@ -194,7 +186,7 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
                  * We must clear the local player's target *before* the call
                  * to changeMap, as it deletes all beings.
                  */
-                player_node->stopAttack();
+                local_player->stopAttack();
 
                 Game *game = Game::instance();
 
@@ -208,21 +200,23 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
                 float scrollOffsetX = 0.0f;
                 float scrollOffsetY = 0.0f;
 
-                /* Scroll if neccessary */
+                /* Scroll if necessary */
+                Map *map = game->getCurrentMap();
+                int tileX = local_player->getTileX();
+                int tileY = local_player->getTileY();
                 if (!sameMap
-                    || (abs(x - player_node->getTileX()) > MAP_TELEPORT_SCROLL_DISTANCE)
-                    || (abs(y - player_node->getTileY()) > MAP_TELEPORT_SCROLL_DISTANCE))
+                    || (abs(x - tileX) > MAP_TELEPORT_SCROLL_DISTANCE)
+                    || (abs(y - tileY) > MAP_TELEPORT_SCROLL_DISTANCE))
                 {
-                    Map *map = game->getCurrentMap();
-                    scrollOffsetX = (x - player_node->getTileX())
-                                    * map->getTileWidth();
-                    scrollOffsetY = (y - player_node->getTileY())
-                                    * map->getTileHeight();
+                    scrollOffsetX = (x - tileX) * map->getTileWidth();
+                    scrollOffsetY = (y - tileY) * map->getTileHeight();
                 }
 
-                player_node->setAction(Being::STAND);
-                player_node->setFrame(0);
-                player_node->setTileCoords(x, y);
+                local_player->setAction(Being::STAND);
+                Vector pos = map->getTileCenter(x, y);
+                local_player->setPosition(pos);
+                // Stop movement
+                local_player->setDestination(pos.x, pos.y);
 
                 logger->log("Adjust scrolling by %d:%d", (int) scrollOffsetX,
                            (int) scrollOffsetY);
@@ -233,7 +227,7 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
 
         case SMSG_PLAYER_STAT_UPDATE_1:
             {
-                if (!player_node)
+                if (!local_player)
                     break;
                 int type = msg.readInt16();
                 int value = msg.readInt32();
@@ -241,20 +235,21 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
                 switch (type)
                 {
                     case 0x0000:
-                      player_node->setWalkSpeed(Vector(value, value, 0));
+                      local_player->setMoveSpeed(Vector(value / 10,
+                                                       value / 10, 0));
                     break;
                     case 0x0004: break; // manner
-                    case 0x0005: player_node->setHp(value); break;
-                    case 0x0006: player_node->setMaxHp(value); break;
-                    case 0x0007: player_node->setMP(value); break;
-                    case 0x0008: player_node->setMaxMP(value); break;
-                    case 0x0009: player_node->setCharacterPoints(value); break;
-                    case 0x000b: player_node->setLevel(value); break;
-                    case 0x000c: player_node->setSkillPoints(value); break;
+                    case 0x0005: PlayerInfo::setAttribute(HP, value); break;
+                    case 0x0006: PlayerInfo::setAttribute(MAX_HP, value); break;
+                    case 0x0007: PlayerInfo::setAttribute(MP, value); break;
+                    case 0x0008: PlayerInfo::setAttribute(MAX_MP, value); break;
+                    case 0x0009: PlayerInfo::setAttribute(CHAR_POINTS, value); break;
+                    case 0x000b: PlayerInfo::setAttribute(LEVEL, value); break;
+                    case 0x000c: PlayerInfo::setAttribute(SKILL_POINTS, value); break;
                     case 0x0018:
-                                 if (value >= player_node->getMaxWeight() / 2 &&
-                                         player_node->getTotalWeight() <
-                                         player_node->getMaxWeight() / 2)
+                                 if (value >= PlayerInfo::getAttribute(MAX_WEIGHT) / 2 &&
+                                         PlayerInfo::getAttribute(TOTAL_WEIGHT) <
+                                         PlayerInfo::getAttribute(MAX_WEIGHT) / 2)
                                  {
                                      weightNotice = new OkDialog(_("Message"),
                                              _("You are carrying more than "
@@ -263,65 +258,44 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
                                      weightNotice->addActionListener(
                                              &weightListener);
                                  }
-                                 player_node->setTotalWeight(value);
+                                 PlayerInfo::setAttribute(TOTAL_WEIGHT, value);
                                  break;
-                    case 0x0019: player_node->setMaxWeight(value); break;
+                    case 0x0019: PlayerInfo::setAttribute(MAX_WEIGHT, value); break;
 
-                    case 0x0029: player_node->setAttributeEffective(ATK, value
-                                                           + ATTR_BONUS(ATK));
-                        player_node->setAttributeBase(ATK, value);
-                        break;
-                    case 0x002a: value += player_node->getAttributeBase(ATK);
-                        player_node->setAttributeEffective(ATK, value); break;
+                    case 0x0029: PlayerInfo::setStatBase(ATK, value); break;
+                    case 0x002a: PlayerInfo::setStatMod(ATK, value); break;
 
-                    case 0x002b: player_node->setAttributeEffective(MATK, value
-                                                           + ATTR_BONUS(MATK));
-                        player_node->setAttributeBase(MATK, value);
-                        if (statusWindow)
-                            statusWindow->update(StatusWindow::MP);
-                        break;
-                    case 0x002c: value += player_node->getAttributeBase(MATK);
-                        player_node->setAttributeEffective(MATK, value);
-                        if (statusWindow)
-                            statusWindow->update(StatusWindow::MP);
-                        break;
-                    case 0x002d: player_node->setAttributeEffective(DEF, value
-                                                           + ATTR_BONUS(DEF));
-                        player_node->setAttributeBase(DEF, value); break;
-                    case 0x002e: value += player_node->getAttributeBase(DEF);
-                        player_node->setAttributeEffective(DEF, value); break;
+                    case 0x002b: PlayerInfo::setStatBase(MATK, value); break;
+                    case 0x002c: PlayerInfo::setStatMod(MATK, value); break;
 
-                    case 0x002f: player_node->setAttributeEffective(MDEF, value
-                                                           + ATTR_BONUS(MDEF));
-                        player_node->setAttributeBase(MDEF, value); break;
-                    case 0x0030: value += player_node->getAttributeBase(MDEF);
-                        player_node->setAttributeEffective(MDEF, value); break;
+                    case 0x002d: PlayerInfo::setStatBase(DEF, value); break;
+                    case 0x002e: PlayerInfo::setStatMod(DEF, value); break;
 
-                    case 0x0031: player_node->setAttributeBase(HIT, value);
-                        player_node->setAttributeEffective(HIT, value); break;
+                    case 0x002f: PlayerInfo::setStatBase(MDEF, value); break;
+                    case 0x0030: PlayerInfo::setStatMod(MDEF, value); break;
 
-                    case 0x0032: player_node->setAttributeEffective(FLEE, value
-                                                           + ATTR_BONUS(FLEE));
-                        player_node->setAttributeBase(FLEE, value); break;
-                    case 0x0033: value += player_node->getAttributeBase(FLEE);
-                        player_node->setAttributeEffective(FLEE, value); break;
+                    case 0x0031: PlayerInfo::setStatBase(HIT, value); break;
 
-                    case 0x0034: player_node->setAttributeBase(CRIT, value);
-                        player_node->setAttributeEffective(CRIT, value); break;
+                    case 0x0032: PlayerInfo::setStatBase(FLEE, value); break;
+                    case 0x0033: PlayerInfo::setStatMod(FLEE, value); break;
 
-                    case 0x0035: player_node->setAttackSpeed(value); break;
-                    case 0x0037: player_node->setAttributeBase(JOB, value);
-                        player_node->setAttributeEffective(JOB, value); break;
-                    case 500: player_node->setGMLevel(value); break;
+                    case 0x0034: PlayerInfo::setStatBase(CRIT, value); break;
+
+                    case 0x0035: local_player->setAttackSpeed(value); break;
+
+                    case 0x0037: PlayerInfo::setStatBase(JOB, value); break;
+
+                    case 500: local_player->setGMLevel(value); break;
                 }
 
-                if (player_node->getHp() == 0 && !deathNotice)
+                if (PlayerInfo::getAttribute(HP) == 0 && !deathNotice)
                 {
+                    viewport->shakeScreen(100);
                     deathNotice = new OkDialog(_("Message"),
                                                randomDeathMessage(),
                                                false);
                     deathNotice->addActionListener(&deathListener);
-                    player_node->setAction(Being::DEAD);
+                    local_player->setAction(Being::DEAD);
                 }
             }
 
@@ -334,35 +308,39 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
             switch (msg.readInt16())
             {
                 case 0x0001:
-                    player_node->setExp(msg.readInt32());
+                    PlayerInfo::setAttribute(EXP, msg.readInt32());
                     break;
                 case 0x0002:
-                    player_node->setExperience(JOB, msg.readInt32(),
-                                    player_node->getExperience(JOB).second);
+                    PlayerInfo::setStatExperience(JOB, msg.readInt32(),
+                                                  PlayerInfo::getStatExperience(JOB).second);
                     break;
+
                 case 0x0014:
                     {
-                        const int curGp = player_node->getMoney();
-                        player_node->setMoney(msg.readInt32());
-                        if (player_node->getMoney() <= curGp)
-                            break;
+                        int oldMoney = PlayerInfo::getAttribute(MONEY);
+                        int newMoney = msg.readInt32();
                         std::string money = Units::formatCurrency(
-                                            player_node->getMoney() - curGp);
-                        if (config.getValue("showpickupchat", 1))
-                            localChatTab->chatLog(strprintf(_("You picked up "
-                                            "%s."), money.c_str()), BY_SERVER);
-                        if (config.getValue("showpickupparticle", 1))
-                            player_node->addMessageToQueue(money,
-                                                    UserPalette::PICKUP_INFO);
+                                            newMoney - oldMoney);
+                        PlayerInfo::setAttribute(MONEY, newMoney);
+                        if (newMoney > oldMoney)
+                        {
+                            if (config.getBoolValue("showpickupchat"))
+                                SERVER_NOTICE(strprintf(_("You picked up %s."),
+                                            Units::formatCurrency(newMoney -
+                                            oldMoney).c_str()))
+                            if (config.getBoolValue("showpickupparticle"))
+                                local_player->addMessageToQueue(money,
+                                                      UserPalette::PICKUP_INFO);
+                        }
                     }
                     break;
                 case 0x0016:
-                    player_node->setExpNeeded(msg.readInt32());
+                    PlayerInfo::setAttribute(EXP_NEEDED, msg.readInt32());
                     break;
                 case 0x0017:
-                    player_node->setExperience(JOB,
-                                    player_node->getExperience(JOB).first,
-                                    msg.readInt32());
+                    PlayerInfo::setStatExperience(JOB,
+                                                  PlayerInfo::getStatExperience(JOB).first,
+                                                  msg.readInt32());
                     break;
             }
             break;
@@ -373,8 +351,8 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
                 int base = msg.readInt32();
                 int bonus = msg.readInt32();
 
-                player_node->setAttributeBase(type, base);
-                player_node->setAttributeEffective(type, base + bonus);
+                PlayerInfo::setStatBase(type, base, false);
+                PlayerInfo::setStatMod(type, bonus);
             }
             break;
 
@@ -386,133 +364,109 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
 
                 if (ok != 1)
                 {
-                    localChatTab->chatLog(_("Cannot raise skill!"),
-                                          BY_SERVER);
+                    SERVER_NOTICE(_("Cannot raise skill!"))
                 }
 
-                int bonus = ATTR_BONUS(type);
-
-                player_node->setAttributeBase(type, value);
-                player_node->setAttributeEffective(type, value + bonus);
+                PlayerInfo::setStatBase(type, value);
             }
             break;
 
         // Updates stats and status points
         case SMSG_PLAYER_STAT_UPDATE_5:
-            player_node->setCharacterPoints(msg.readInt16());
+            PlayerInfo::setAttribute(CHAR_POINTS, msg.readInt16());
 
             {
                 int val = msg.readInt8();
-                player_node->setAttributeEffective(STR, val + ATTR_BONUS(STR));
-                player_node->setAttributeBase(STR, val);
+                PlayerInfo::setStatBase(STRENGTH, val);
                 if (val >= 99)
                 {
-                    statusWindow->setPointsNeeded(STR, 0);
+                    statusWindow->setPointsNeeded(STRENGTH, 0);
                     msg.readInt8();
                 }
                 else
                 {
-                    statusWindow->setPointsNeeded(STR, msg.readInt8());
+                    statusWindow->setPointsNeeded(STRENGTH, msg.readInt8());
                 }
 
                 val = msg.readInt8();
-                player_node->setAttributeEffective(AGI, val + ATTR_BONUS(AGI));
-                player_node->setAttributeBase(AGI, val);
+                PlayerInfo::setStatBase(AGILITY, val);
                 if (val >= 99)
                 {
-                    statusWindow->setPointsNeeded(AGI, 0);
+                    statusWindow->setPointsNeeded(AGILITY, 0);
                     msg.readInt8();
                 }
                 else
                 {
-                    statusWindow->setPointsNeeded(AGI, msg.readInt8());
+                    statusWindow->setPointsNeeded(AGILITY, msg.readInt8());
                 }
 
                 val = msg.readInt8();
-                player_node->setAttributeEffective(VIT, val + ATTR_BONUS(VIT));
-                player_node->setAttributeBase(VIT, val);
+                PlayerInfo::setStatBase(VITALITY, val);
                 if (val >= 99)
                 {
-                    statusWindow->setPointsNeeded(VIT, 0);
+                    statusWindow->setPointsNeeded(VITALITY, 0);
                     msg.readInt8();
                 }
                 else
                 {
-                    statusWindow->setPointsNeeded(VIT, msg.readInt8());
+                    statusWindow->setPointsNeeded(VITALITY, msg.readInt8());
                 }
 
                 val = msg.readInt8();
-                player_node->setAttributeEffective(INT, val + ATTR_BONUS(INT));
-                player_node->setAttributeBase(INT, val);
+                PlayerInfo::setStatBase(INTELLIGENCE, val);
                 if (val >= 99)
                 {
-                    statusWindow->setPointsNeeded(INT, 0);
+                    statusWindow->setPointsNeeded(INTELLIGENCE, 0);
                     msg.readInt8();
                 }
                 else
                 {
-                    statusWindow->setPointsNeeded(INT, msg.readInt8());
+                    statusWindow->setPointsNeeded(INTELLIGENCE, msg.readInt8());
                 }
 
                 val = msg.readInt8();
-                player_node->setAttributeEffective(DEX, val + ATTR_BONUS(DEX));
-                player_node->setAttributeBase(DEX, val);
+                PlayerInfo::setStatBase(DEXTERITY, val);
                 if (val >= 99)
                 {
-                    statusWindow->setPointsNeeded(DEX, 0);
+                    statusWindow->setPointsNeeded(DEXTERITY, 0);
                     msg.readInt8();
                 }
                 else
                 {
-                    statusWindow->setPointsNeeded(DEX, msg.readInt8());
+                    statusWindow->setPointsNeeded(DEXTERITY, msg.readInt8());
                 }
 
                 val = msg.readInt8();
-                player_node->setAttributeEffective(LUK, val + ATTR_BONUS(LUK));
-                player_node->setAttributeBase(LUK, val);
+                PlayerInfo::setStatBase(LUCK, val);
                 if (val >= 99)
                 {
-                    statusWindow->setPointsNeeded(LUK, 0);
+                    statusWindow->setPointsNeeded(LUCK, 0);
                     msg.readInt8();
                 }
                 else
                 {
-                    statusWindow->setPointsNeeded(LUK, msg.readInt8());
+                    statusWindow->setPointsNeeded(LUCK, msg.readInt8());
                 }
 
-                val = msg.readInt16(); // ATK
-                player_node->setAttributeBase(ATK, val);
-                val += msg.readInt16();  // ATK bonus
-                player_node->setAttributeEffective(ATK, val);
+                PlayerInfo::setStatBase(ATK, msg.readInt16(), false);
+                PlayerInfo::setStatMod(ATK, msg.readInt16());
 
-                val = msg.readInt16(); // MATK
-                player_node->setAttributeBase(MATK, val);
-                val += msg.readInt16();  // MATK bonus
-                player_node->setAttributeEffective(MATK, val);
-                statusWindow->update(StatusWindow::MP);
+                PlayerInfo::setStatBase(MATK, msg.readInt16(), false);
+                PlayerInfo::setStatMod(MATK, msg.readInt16());
 
-                val = msg.readInt16(); // DEF
-                player_node->setAttributeBase(DEF, val);
-                val += msg.readInt16();  // DEF bonus
-                player_node->setAttributeEffective(DEF, val);
 
-                val = msg.readInt16(); // MDEF
-                player_node->setAttributeBase(MDEF, val);
-                val += msg.readInt16();  // MDEF bonus
-                player_node->setAttributeEffective(MDEF, val);
+                PlayerInfo::setStatBase(DEF, msg.readInt16(), false);
+                PlayerInfo::setStatMod(DEF, msg.readInt16());
 
-                val = msg.readInt16(); // HIT
-                player_node->setAttributeBase(HIT, val);
-                player_node->setAttributeEffective(HIT, val);
+                PlayerInfo::setStatBase(MDEF, msg.readInt16(), false);
+                PlayerInfo::setStatMod(MDEF, msg.readInt16());
 
-                val = msg.readInt16(); // FLEE
-                player_node->setAttributeBase(FLEE, val);
-                val += msg.readInt16();  // FLEE bonus
-                player_node->setAttributeEffective(FLEE, val);
+                PlayerInfo::setStatBase(HIT, msg.readInt16());
 
-                val = msg.readInt16();
-                player_node->setAttributeBase(CRIT, val);
-                player_node->setAttributeEffective(CRIT, val);
+                PlayerInfo::setStatBase(FLEE, msg.readInt16(), false);
+                PlayerInfo::setStatMod(FLEE, msg.readInt16());
+
+                PlayerInfo::setStatBase(CRIT, msg.readInt16());
             }
 
             msg.readInt16();  // manner
@@ -522,22 +476,22 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
             switch (msg.readInt16())
             {
                 case 0x0020:
-                    statusWindow->setPointsNeeded(STR, msg.readInt8());
+                    statusWindow->setPointsNeeded(STRENGTH, msg.readInt8());
                     break;
                 case 0x0021:
-                    statusWindow->setPointsNeeded(AGI, msg.readInt8());
+                    statusWindow->setPointsNeeded(AGILITY, msg.readInt8());
                     break;
                 case 0x0022:
-                    statusWindow->setPointsNeeded(VIT, msg.readInt8());
+                    statusWindow->setPointsNeeded(VITALITY, msg.readInt8());
                     break;
                 case 0x0023:
-                    statusWindow->setPointsNeeded(INT, msg.readInt8());
+                    statusWindow->setPointsNeeded(INTELLIGENCE, msg.readInt8());
                     break;
                 case 0x0024:
-                    statusWindow->setPointsNeeded(DEX, msg.readInt8());
+                    statusWindow->setPointsNeeded(DEXTERITY, msg.readInt8());
                     break;
                 case 0x0025:
-                    statusWindow->setPointsNeeded(LUK, msg.readInt8());
+                    statusWindow->setPointsNeeded(LUCK, msg.readInt8());
                     break;
             }
             break;
@@ -549,8 +503,9 @@ void PlayerHandler::handleMessage(Net::MessageIn &msg)
                 switch (type)
                 {
                     case 0:
-                        localChatTab->chatLog(_("Equip arrows first."),
-                                             BY_SERVER);
+                        {
+                            SERVER_NOTICE(_("Equip arrows first."))
+                        }
                         break;
                     default:
                         logger->log("0x013b: Unhandled message %i", type);
@@ -576,7 +531,7 @@ void PlayerHandler::emote(int emoteId)
 
 void PlayerHandler::increaseAttribute(int attr)
 {
-    if (attr >= STR && attr <= LUK)
+    if (attr >= STRENGTH && attr <= LUCK)
     {
         MessageOut outMsg(CMSG_STAT_UPDATE_REQUEST);
         outMsg.writeInt16(attr);
@@ -591,7 +546,7 @@ void PlayerHandler::decreaseAttribute(int attr)
 
 void PlayerHandler::increaseSkill(int skillId)
 {
-    if (player_node->getSkillPoints() <= 0)
+    if (PlayerInfo::getAttribute(SKILL_POINTS) <= 0)
         return;
 
     MessageOut outMsg(CMSG_SKILL_LEVELUP_REQUEST);
@@ -601,11 +556,15 @@ void PlayerHandler::increaseSkill(int skillId)
 void PlayerHandler::pickUp(FloorItem *floorItem)
 {
     static Uint32 lastTime = 0;
-    if (SDL_GetTicks() < lastTime + 100)
+
+    // Avoid spamming the server with pick-up requests to prevent the player
+    // from being kicked.
+    if (!floorItem || SDL_GetTicks() < lastTime + 100)
         return;
 
     MessageOut outMsg(CMSG_ITEM_PICKUP);
     outMsg.writeInt32(floorItem->getId());
+
     lastTime = SDL_GetTicks();
 }
 
@@ -618,8 +577,12 @@ void PlayerHandler::setDirection(char direction)
 
 void PlayerHandler::setDestination(int x, int y, int direction)
 {
+    // The destination coordinates are received in pixel, so we translate them
+    // into tiles.
+    Map *map = Game::instance()->getCurrentMap();
     MessageOut outMsg(CMSG_PLAYER_CHANGE_DEST);
-    outMsg.writeCoordinates(x, y, direction);
+    outMsg.writeCoordinates(x / map->getTileWidth(), y / map->getTileHeight(),
+                            direction);
 }
 
 void PlayerHandler::changeAction(Being::Action action)
@@ -655,7 +618,7 @@ void PlayerHandler::ignoreAll(bool ignore)
 
 bool PlayerHandler::canUseMagic()
 {
-    return player_node->getAttributeEffective(MATK) > 0;
+    return PlayerInfo::getStatEffective(MATK) > 0;
 }
 
 bool PlayerHandler::canCorrectAttributes()
@@ -668,11 +631,35 @@ int PlayerHandler::getJobLocation()
     return JOB;
 }
 
-Vector PlayerHandler::getDefaultWalkSpeed()
+Vector PlayerHandler::getDefaultMoveSpeed() const
 {
     // Return an normalized speed for any side
     // as the offset is calculated elsewhere.
-    return Vector(150, 150, 0);
+    // in ticks per tile.
+    return Vector(15.0f, 15.0f, 0.0f);
+}
+
+Vector PlayerHandler::getPixelsPerTickMoveSpeed(const Vector &speed, Map *map)
+{
+    Game *game = Game::instance();
+
+    if (game && !map)
+        map = game->getCurrentMap();
+
+    if (!map || speed.x == 0 || speed.y == 0)
+    {
+        logger->log("TmwAthena::PlayerHandler: Speed set to default: "
+                    "Map not yet initialized or invalid speed.");
+        return getDefaultMoveSpeed();
+    }
+
+    Vector speedInTicks;
+
+    // speedInTicks.z = 0; // We don't use z for now.
+    speedInTicks.x = 1 / speed.x * (float)map->getTileWidth();
+    speedInTicks.y = 1 / speed.y * (float)map->getTileHeight();
+
+    return speedInTicks;
 }
 
 } // namespace TmwAthena
