@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -21,13 +21,12 @@
 
 #include "net/tmwa/specialhandler.h"
 
-#include "localplayer.h"
+#include "event.h"
 #include "log.h"
+#include "playerinfo.h"
 
 #include "gui/skilldialog.h"
 
-#include "gui/widgets/chattab.h"
-
 #include "net/messagein.h"
 #include "net/messageout.h"
 
@@ -105,8 +104,7 @@ void SpecialHandler::handleMessage(Net::MessageIn &msg)
                 msg.skip(24); // unused
                 int up = msg.readInt8();
 
-                player_node->setAttributeBase(skillId, level);
-                player_node->setAttributeEffective(skillId, level);
+                PlayerInfo::setStatBase(skillId, level);
                 if (skillDialog)
                     skillDialog->setModifiable(skillId, up);
             }
@@ -120,8 +118,7 @@ void SpecialHandler::handleMessage(Net::MessageIn &msg)
                 msg.readInt16(); // range
                 int up = msg.readInt8();
 
-                player_node->setAttributeBase(skillId, level);
-                player_node->setAttributeEffective(skillId, level);
+                PlayerInfo::setStatBase(skillId, level);
                 skillDialog->setModifiable(skillId, up);
             }
             break;
@@ -219,7 +216,7 @@ void SpecialHandler::handleMessage(Net::MessageIn &msg)
                 }
             }
 
-            localChatTab->chatLog(msg);
+            SERVER_NOTICE(msg)
             break;
     }
 }
