@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -19,7 +19,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "player.h"
+#include "being.h"
 
 #ifndef NET_TA_TOKEN_H
 #define NET_TA_TOKEN_H
