@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -51,6 +51,9 @@ class TradeHandler : public MessageHandler, public Net::TradeHandler
         void finish();
 
         void cancel();
+
+    private:
+        bool mTrading;
 };
 
 } // namespace TmwAthena
