@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -25,8 +25,6 @@
 
 #include "resources/image.h"
 
-#include "utils/stringutils.h"
-
 #ifdef USE_OPENGL
 
 #ifdef __APPLE__
@@ -79,7 +77,17 @@ bool OpenGLGraphics::setVideoMode(int w, int h, int bpp, bool fs, bool hwaccel)
     mHWAccel = hwaccel;
 
     if (fs)
+    {
         displayFlags |= SDL_FULLSCREEN;
+    }
+    else
+    {
+        // Resizing currently not supported on Windows, where it would require
+        // reuploading all textures.
+#if !defined(_WIN32)
+        displayFlags |= SDL_RESIZABLE;
+#endif
+    }
 
     SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
 
@@ -656,12 +664,14 @@ void OpenGLGraphics::_beginDraw()
 
 void OpenGLGraphics::_endDraw()
 {
+    popClipArea();
 }
 
 SDL_Surface* OpenGLGraphics::getScreenshot()
 {
     int h = mTarget->h;
     int w = mTarget->w;
+    GLint pack = 1;
 
     SDL_Surface *screenshot = SDL_CreateRGBSurface(
             SDL_SWSURFACE,
@@ -672,6 +682,7 @@ SDL_Surface* OpenGLGraphics::getScreenshot()
         SDL_LockSurface(screenshot);
 
     // Grap the pixel buffer and write it to the SDL surface
+    glGetIntegerv(GL_PACK_ALIGNMENT, &pack);
     glPixelStorei(GL_PACK_ALIGNMENT, 1);
     glReadPixels(0, 0, w, h, GL_RGB, GL_UNSIGNED_BYTE, screenshot->pixels);
 
@@ -691,6 +702,8 @@ SDL_Surface* OpenGLGraphics::getScreenshot()
 
     free(buf);
 
+    glPixelStorei(GL_PACK_ALIGNMENT, pack);
+
     if (SDL_MUSTLOCK(screenshot))
         SDL_UnlockSurface(screenshot);
 
