@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -105,10 +105,8 @@ Image::~Image()
     unload();
 }
 
-Resource *Image::load(void *buffer, unsigned bufferSize)
+Resource *Image::load(SDL_RWops *rw)
 {
-    // Load the raw file data from the buffer in an RWops structure
-    SDL_RWops *rw = SDL_RWFromMem(buffer, bufferSize);
     SDL_Surface *tmpImage = IMG_Load_RW(rw, 1);
 
     if (!tmpImage)
@@ -123,9 +121,8 @@ Resource *Image::load(void *buffer, unsigned bufferSize)
     return image;
 }
 
-Resource *Image::load(void *buffer, unsigned bufferSize, Dye const &dye)
+Resource *Image::load(SDL_RWops *rw, Dye const &dye)
 {
-    SDL_RWops *rw = SDL_RWFromMem(buffer, bufferSize);
     SDL_Surface *tmpImage = IMG_Load_RW(rw, 1);
 
     if (!tmpImage)
@@ -329,109 +326,52 @@ void Image::setAlpha(float alpha)
     }
 }
 
-Image* Image::SDLmerge(Image *image, int x, int y)
+Image *Image::SDLgetScaledImage(int width, int height)
 {
-    if (!mSDLSurface)
-        return NULL;
+    if (width == 0 || height == 0)
+        return 0;
 
-    SDL_Surface* surface = new SDL_Surface(*(image->mSDLSurface));
-
-    Uint32 surface_pix, cur_pix;
-    Uint8 r, g, b, a, p_r, p_g, p_b, p_a;
-    double f_a, f_ca, f_pa;
-    SDL_PixelFormat *current_fmt = mSDLSurface->format;
-    SDL_PixelFormat *surface_fmt = surface->format;
-    int current_offset, surface_offset;
-    int offset_x, offset_y;
-
-    SDL_LockSurface(surface);
-    SDL_LockSurface(mSDLSurface);
-    // for each pixel lines of a source image
-    for (offset_x = (x > 0 ? 0 : -x); offset_x < image->getWidth() &&
-                     x + offset_x < getWidth(); offset_x++)
+    // Increase our reference count and return ourselves in case of same size
+    if (width == getWidth() && height == getHeight())
     {
-        for (offset_y = (y > 0 ? 0 : -y); offset_y < image->getHeight()
-                        && y + offset_y < getHeight(); offset_y++)
+        incRef();
+        return this;
+    }
+
+    if (!mSDLSurface)
+        return 0;
+
+    ResourceManager *resman = ResourceManager::getInstance();
+
+    // Generate a unique ID path for storing the scaled version in the
+    // resource manager.
+    std::string idPath = getIdPath();
+    idPath += ":scaled:";
+    idPath += toString(width);
+    idPath += "x";
+    idPath += toString(height);
+
+    // Try whether a scaled version is already available
+    Image *scaledImage = static_cast<Image*>(resman->get(idPath));
+
+    if (!scaledImage)
+    {
+        // No scaled version with this size exists already, so create one
+        SDL_Surface *scaledSurface = zoomSurface(mSDLSurface,
+                                                 (double) width / getWidth(),
+                                                 (double) height / getHeight(),
+                                                 1);
+
+        if (scaledSurface)
         {
-            // Computing offset on both images
-            current_offset = (y + offset_y) * getWidth() + x + offset_x;
-            surface_offset = offset_y * surface->w + offset_x;
+            scaledImage = load(scaledSurface);
+            SDL_FreeSurface(scaledSurface);
 
-            // Retrieving a pixel to merge
-            surface_pix = ((Uint32*) surface->pixels)[surface_offset];
-            cur_pix = ((Uint32*) mSDLSurface->pixels)[current_offset];
-
-            // Retreiving each channel of the pixel using pixel format
-            r = (Uint8)(((surface_pix & surface_fmt->Rmask) >>
-                          surface_fmt->Rshift) << surface_fmt->Rloss);
-            g = (Uint8)(((surface_pix & surface_fmt->Gmask) >>
-                          surface_fmt->Gshift) << surface_fmt->Gloss);
-            b = (Uint8)(((surface_pix & surface_fmt->Bmask) >>
-                          surface_fmt->Bshift) << surface_fmt->Bloss);
-            a = (Uint8)(((surface_pix & surface_fmt->Amask) >>
-                          surface_fmt->Ashift) << surface_fmt->Aloss);
-
-            // Retreiving previous alpha value
-            p_a = (Uint8)(((cur_pix & current_fmt->Amask) >>
-                            current_fmt->Ashift) << current_fmt->Aloss);
-
-            // new pixel with no alpha or nothing on previous pixel
-            if (a == SDL_ALPHA_OPAQUE || (p_a == 0 && a > 0))
-                ((Uint32 *)(surface->pixels))[current_offset] =
-                    SDL_MapRGBA(current_fmt, r, g, b, a);
-            else if (a > 0)
-            { // alpha is lower => merge color with previous value
-                f_a = (double) a / 255.0;
-                f_ca = 1.0 - f_a;
-                f_pa = (double) p_a / 255.0;
-                p_r = (Uint8)(((cur_pix & current_fmt->Rmask) >>
-                                current_fmt->Rshift) << current_fmt->Rloss);
-                p_g = (Uint8)(((cur_pix & current_fmt->Gmask) >>
-                                current_fmt->Gshift) << current_fmt->Gloss);
-                p_b = (Uint8)(((cur_pix & current_fmt->Bmask) >>
-                                current_fmt->Bshift) << current_fmt->Bloss);
-                r = (Uint8)((double) p_r * f_ca * f_pa + (double)r * f_a);
-                g = (Uint8)((double) p_g * f_ca * f_pa + (double)g * f_a);
-                b = (Uint8)((double) p_b * f_ca * f_pa + (double)b * f_a);
-                a = (a > p_a ? a : p_a);
-               ((Uint32 *)(surface->pixels))[current_offset] =
-                   SDL_MapRGBA(current_fmt, r, g, b, a);
-            }
+            // Place the scaled image in the resource manager
+            resman->addResource(idPath, scaledImage);
         }
     }
-    SDL_UnlockSurface(surface);
-    SDL_UnlockSurface(mSDLSurface);
 
-    Image *newImage = new Image(surface);
-
-    return newImage;
-}
-
-Image* Image::SDLgetScaledImage(int width, int height)
-{
-    // No scaling on incorrect new values.
-    if (width == 0 || height == 0)
-    return NULL;
-
-    // No scaling when there is ... no different given size ...
-    if (width == getWidth() && height == getHeight())
-        return NULL;
-
-    Image* scaledImage = NULL;
-    SDL_Surface* scaledSurface = NULL;
-
-    if (mSDLSurface)
-    {
-        scaledSurface = zoomSurface(mSDLSurface,
-                    (double) width / getWidth(),
-                    (double) height / getHeight(),
-                    1);
-
-        // The load function takes care of the SDL<->OpenGL implementation
-        // and about freeing the given SDL_surface*.
-        if (scaledSurface)
-            scaledImage = load(scaledSurface);
-    }
     return scaledImage;
 }
 
@@ -556,8 +496,6 @@ Image *Image::_GLload(SDL_Surface *tmpImage)
                 tmpImage->pixels);
 
         glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-        glTexParameteri(mTextureType, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-        glTexParameteri(mTextureType, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 
         if (SDL_MUSTLOCK(tmpImage))
             SDL_UnlockSurface(tmpImage);
