@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -185,11 +185,9 @@ Map *MapReader::readMap(xmlNodePtr node, const std::string &path)
                         if (config.getValue("showWarps", 1))
                         {
                             map->addParticleEffect(
-                            paths.getValue("particles",
-                                           "graphics/particles/")
-                            + paths.getValue("portalEffectFile",
-                                             "warparea.particle.xml"),
-                                objX, objY, objW, objH);
+                                     paths.getStringValue("particles")
+                                     + paths.getStringValue("portalEffectFile"),
+                                                   objX, objY, objW, objH);
                         }
                     }
                     else
@@ -222,8 +220,19 @@ void MapReader::readProperties(xmlNodePtr node, Properties *props)
     }
 }
 
-static void setTile(Map *map, MapLayer *layer, int x, int y, int gid)
+static void setTile(Map *map, MapLayer *layer, int x, int y, unsigned gid)
 {
+    // Bits on the far end of the 32-bit global tile ID are used for tile flags
+    const int FlippedHorizontallyFlag   = 0x80000000;
+    const int FlippedVerticallyFlag     = 0x40000000;
+    const int FlippedAntiDiagonallyFlag = 0x20000000;
+
+    // Clear the flags
+    // TODO: It would be nice to properly support these flags later
+    gid &= ~(FlippedHorizontallyFlag |
+             FlippedVerticallyFlag |
+             FlippedAntiDiagonallyFlag);
+
     const Tileset * const set = map->getTilesetWithGid(gid);
     if (layer)
     {
@@ -256,7 +265,7 @@ void MapReader::readLayer(xmlNodePtr node, Map *map)
 
     if (!isCollisionLayer)
     {
-        layer = new MapLayer(offsetX, offsetY, w, h, isFringeLayer);
+        layer = new MapLayer(offsetX, offsetY, w, h, isFringeLayer, map);
         map->addLayer(layer);
     }
 
@@ -335,7 +344,7 @@ void MapReader::readLayer(xmlNodePtr node, Map *map)
 
                 for (int i = 0; i < binLen - 3; i += 4)
                 {
-                    const int gid = binData[i] |
+                    const unsigned gid = binData[i] |
                         binData[i + 1] << 8 |
                         binData[i + 2] << 16 |
                         binData[i + 3] << 24;
@@ -377,7 +386,7 @@ void MapReader::readLayer(xmlNodePtr node, Map *map)
             {
                 pos = csv.find_first_of(",", oldPos);
 
-                const int gid = atoi(csv.substr(oldPos, pos - oldPos).c_str());
+                unsigned gid = atol(csv.substr(oldPos, pos - oldPos).c_str());
 
                 setTile(map, layer, x, y, gid);
 
@@ -402,7 +411,7 @@ void MapReader::readLayer(xmlNodePtr node, Map *map)
                 if (!xmlStrEqual(childNode2->name, BAD_CAST "tile"))
                     continue;
 
-                const int gid = XML::getProperty(childNode2, "gid", -1);
+                unsigned gid = XML::getProperty(childNode2, "gid", 0);
                 setTile(map, layer, x, y, gid);
 
                 x++;
@@ -428,7 +437,9 @@ void MapReader::readLayer(xmlNodePtr node, Map *map)
 Tileset *MapReader::readTileset(xmlNodePtr node, const std::string &path,
                                 Map *map)
 {
-    int firstGid = XML::getProperty(node, "firstgid", 0);
+    unsigned firstGid = XML::getProperty(node, "firstgid", 0);
+    int margin = XML::getProperty(node, "margin", 0);
+    int spacing = XML::getProperty(node, "spacing", 0);
     XML::Document* doc = NULL;
     Tileset *set = NULL;
     std::string pathDir(path);
@@ -463,7 +474,8 @@ Tileset *MapReader::readTileset(xmlNodePtr node, const std::string &path,
 
                 if (tilebmp)
                 {
-                    set = new Tileset(tilebmp, tw, th, firstGid);
+                    set = new Tileset(tilebmp, tw, th, firstGid, margin,
+                                      spacing);
                     tilebmp->decRef();
                 }
                 else
