@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2008-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -22,23 +22,7 @@
 #ifndef NPC_DB_H
 #define NPC_DB_H
 
-#include <list>
-#include <map>
-#include <string>
-
-struct NPCsprite
-{
-    std::string sprite;
-    int variant;
-};
-
-struct NPCInfo
-{
-    std::list<NPCsprite*> sprites;
-    std::list<std::string> particles;
-};
-
-typedef std::map<int, NPCInfo*> NPCInfos;
+class BeingInfo;
 
 /**
  * NPC information database.
@@ -49,9 +33,7 @@ namespace NPCDB
 
     void unload();
 
-    const NPCInfo& get(int id);
-
-    typedef NPCInfos::iterator NPCInfosIterator;
+    BeingInfo *get(int id);
 }
 
 #endif
