@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -27,16 +27,7 @@
 
 #include <cassert>
 
-Resource::~Resource()
-{
-}
-
-void Resource::incRef()
-{
-    mRefCount++;
-}
-
-void Resource::decRef()
+void Resource::decRef(OrphanPolicy orphanPolicy)
 {
     // Reference may not already have reached zero
     if (mRefCount == 0) {
@@ -44,12 +35,22 @@ void Resource::decRef()
         assert(false);
     }
 
-    mRefCount--;
+    --mRefCount;
 
     if (mRefCount == 0)
     {
-        // Warn the manager that this resource is no longer used.
         ResourceManager *resman = ResourceManager::getInstance();
-        resman->release(this);
+
+        switch (orphanPolicy)
+        {
+            case DeleteLater:
+            default:
+                resman->release(this);
+                break;
+            case DeleteImmediately:
+                resman->remove(this);
+                delete this;
+                break;
+        }
     }
 }
