@@ -1,7 +1,7 @@
 /*
  *  The Mana Client
  *  Copyright (C) 2004-2009  The Mana World Development Team
- *  Copyright (C) 2009-2010  The Mana Developers
+ *  Copyright (C) 2009-2012  The Mana Developers
  *
  *  This file is part of The Mana Client.
  *
@@ -21,15 +21,20 @@
 
 #include "utils/xml.h"
 
+#include <iostream>
+#include <fstream>
+#include <cstring>
+
+#include <libxml/parser.h>
+#include <libxml/xmlerror.h>
+
 #include "log.h"
 
 #include "resources/resourcemanager.h"
 
+#include "utils/stringutils.h"
 #include "utils/zlib.h"
 
-#include <libxml/parser.h>
-#include <libxml/xmlerror.h>
-
 namespace XML
 {
     static void xmlLogger(void *ctx, xmlErrorPtr error);
@@ -43,10 +48,10 @@ namespace XML
     Document::Document(const std::string &filename, bool useResman):
         mDoc(0)
     {
-        XMLContext *ctx = new XMLContext();
-        ctx->file = filename;
-        ctx->resman = useResman;
-        xmlSetStructuredErrorFunc(ctx, xmlLogger);
+        XMLContext ctx;
+        ctx.file = filename;
+        ctx.resman = useResman;
+        xmlSetStructuredErrorFunc(&ctx, xmlLogger);
 
         int size;
         char *data = NULL;
@@ -94,7 +99,7 @@ namespace XML
         xmlChar *prop = xmlGetProp(node, BAD_CAST name);
         if (prop)
         {
-            ret = atoi((char*)prop);
+            ret = atol((char*)prop);
             xmlFree(prop);
         }
 
@@ -129,6 +134,18 @@ namespace XML
         return def;
     }
 
+    bool getBoolProperty(xmlNodePtr node, const char* name, bool def)
+    {
+        bool ret = def;
+        xmlChar *prop = xmlGetProp(node, BAD_CAST name);
+        if (prop)
+        {
+            ret = getBoolFromString((char*) prop, def);
+            xmlFree(prop);
+        }
+        return ret;
+    }
+
     xmlNodePtr findFirstChildByName(xmlNodePtr parent, const char *name)
     {
         for_each_xml_child_node(child, parent)
